#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@File    : le.com.py
@Time    : 2022/2/13 下午6:13  
"""
import template


class Main(template.Template):
    def __init__(self):
        super().__init__()
        self.title = "乐视视频(LE)"

    def query(self):
        p = self.params
        if self.hasurl(p["parse"]):
            vid = self.match("\/(\d+)", p["parse"])
        else:
            vid = p["parse"]
        if p.get("query"):
            key = self.getKey(self.time)
            playParams = {
                "platid": "1",
                "splatid": "105",
                "tss": "ios",
                "id": vid,
                "detect": "0",
                "dvtype": "1000",
                "accessyx": "1",
                "domain": "m.le.com",
                "tkey": key,
                "devid": "6E622B75C984100A4495B527DCD5FB56F585580C",
                "source": "1001",
                "lang": "en",
                "region": "cn",
                "isHttps": "0",
            }
            content = self.curl(
                {
                    "url": "http://player-pc.le.com/mms/out/video/playJson.json",
                    "params": playParams,
                }
            )
            self.logging.debug(f"playJson: {content}")
            data = self.loads(content)
            if self.haskey(data, "msgs.playurl"):
                mid = data["msgs"]["playurl"]["mid"][1:-1]
                title = data["msgs"]["playurl"]["title"]
                image = sorted(data["msgs"]["playurl"]["picAll"].values())[-1]

        return self.compact()

    def parse(self):

        p = self.params
        assert p["vid"], "vid"
        pcode = "010210000"
        url = f"http://t.api.mob.app.letv.com/play?tm={self.time}&playid=0&tss=ios&pcode={pcode}&version=6.0&pid=93327&vid={p['vid']}&v=android&res=json&_debug=1"
        html = self.curl(url)
        self.logging.debug(f"getPlaySource: {html} \r\n")

        playJson = self.loads(html)
        assert self.haskey(playJson, "header.status", "1"), "data"
        ary = ["mp4", "180", "350", "1000", "800", "1300", "720p", "1080p", "1080p3m"]
        dispatch = playJson["body"]["videofile"]["infos"]
        lists = dict([[i.replace("mp4_", ""), dispatch[i]] for i in dispatch])
        quality = list(lists.keys())
        quality.sort(key=ary.index)
        show = self.data(quality, p["hd"])
        size = lists[show]["filesize"]
        m3u8Url = lists[show]["mainUrl"]
        mp4 = self.curl(
            {"url": m3u8Url.replace("tss=ios", "tss=no"), "response": "json"}
        )["location"]
        title = playJson["body"]["videoInfo"]["nameCn"]
        duration = playJson["body"]["videoInfo"]["duration"]
        image = list(playJson["body"]["videoInfo"]["picAll"].values())[-1]
        extra = {
            "headers": {
                "User-Agent": "LetvIphoneClient/9.23.3 (iPhone; iOS 13.7; Scale/2.00)",
                "Accept-Language": "zh-Hans-CN;q=1, en-CN;q=0.9",
            }
        }
        return self.compact()

    def getKey(self, t):
        for s in range(0, 8):
            e = 1 & t
            t >>= 1
            e <<= 31
            t += e
        return t ^ 185025305



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@File    : qq.py
@Time    : 2022/9/8 下午10:51  
"""
import template


class Main(template.Template):
    def __init__(self):
        super().__init__()
        self.title = "腾讯剧集列表"

    def videoList(self, params):
        if params["parse"].startswith("http"):
            cover = self.match("cover\/(\w+)", params["parse"])
        else:
            cover = params["parse"]

        s = self.curl(
            {
                "url": "https://pbaccess.video.qq.com/trpc.universal_backend_service.page_server_rpc.PageServer/GetPageData?video_appid=3000010&vplatform=2",
                "json": {
                    "page_params": {
                        "page_type": "detail_operation",
                        "page_id": "vsite_episode_list",
                        "id_type": "1",
                        "page_size": "50",
                        "cid": cover,
                        "vid": "",
                        "lid": "0",
                        "req_from": "web_mobile",
                        "page_context": "",
                    }
                },
                "cookie": {
                    "sd_cookie_crttime": "165235985518",
                    "vversion_name": "8.2.95",
                    "video_bucketid": "4",
                    "appid": "wxa75efa648b60994b",
                    "_video_qq_appid": "wxa75efa648b60994",
                },
                "headers": {
                    "Host": "pbaccess.video.qq.com",
                    "user-agent": "Mozilla/5.0 (iPhone; CPU iPhone OS 11_4 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15F79;supportJDSHWK/1;",
                    "accept": "application/json",
                    "accept-language": "zh-CN,zh;q=0.8,zh-TW;q=0.7,zh-HK;q=0.5,en-US;q=0.3,en;q=0.2",
                    "referer": "https://m.v.qq.com/",
                    "content-type": "application/json;charset=UTF-8",
                    "origin": "https://m.v.qq.com",
                    "sec-fetch-dest": "empty",
                    "sec-fetch-mode": "cors",
                    "sec-fetch-site": "same-site",
                    "te": "trailers",
                },
            }
        )
        data = self.jsonParse(s)
        playList = []

        for k in self.haskey(
            data, "data.module_list_datas.0.module_datas.0.item_data_lists.item_datas"
        ):
            playList.append(
                {
                    "parse": k["item_params"]["vid"],
                    "title": k["item_params"]["play_title"],
                }
            )
        return {"data": playList, "category": "video", "type": "qq"}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@File    : mgtv.py
@Time    : 2022/8/30 上午2:08  
"""
import template


class Main(template.Template):
    def __init__(self):
        super().__init__()
        self.title = "芒果视频(MGTV)"

    def query(self):
        p = self.params
        if self.hasurl(p["parse"]):
            vid = self.match("\/(\d+)\.", p["parse"])
        else:
            vid = p["parse"]

        if p.get("query"):
            url = f"https://pcweb.api.mgtv.com/video/info?vid={vid}"
            html = self.curl(url)
            self.logging.debug(f"getInfo: {html} \r\n")
            json = self.loads(html)
            if self.haskey(json, "data.info"):
                info = json["data"]["info"]
                title = info["videoName"]
                image = info["videoImage"]
                duration = self.seconds(info["time"])
        return self.compact()

    def parse(self):
        p = self.params

        timestamp = self.timestamp
        assert p["vid"], "vid"
        vid = p["vid"]

        params = {
            "abroad": "0",
            "appVersion": "5.5.1",
            "clipId": "",
            "device": "iPhone",
            "dname": "abcdefg",
            "guid": "898839756384243712",
            "keepPlay": "1",
            "localPlayVideoId": "0",
            "localVideoWatchTime": "0",
            "mac": "e4b87ad76c79eb1183bfffcbcda3887ea6e819e2",
            "osType": "ios",
            "osVersion": "9.3.2",
            "seqId": "436ee82f7f5027eb051c7f402235d1b8",
            "source": "1",
            "ticket": "",
            "videoId": vid,
        }
        url = "http://mobile-bjyg.api.mgtv.com/v7/video/getSource?{}".format(
            self.urlencode(params)
        )

        html = self.curl(
            {
                "url": "http://mobile-bjyg.api.mgtv.com/v7/video/getSource",
                "params": params,
                "headers": {
                    "useragent": "MGTV-iPhone-appstore/5.5.1 (iPhone; iOS 9.3.2; Scale/2.00)",
                    "encoding": "utf8",
                },
            }
        )
        self.logging.debug(f"getSource: {html} \r\n")
        data = self.loads(html)


        assert data["data"], data["msg"]

        adparams = self.loads(data["data"]["adParams"])

        pay = adparams["v"]["ispay"]
        title = data["data"]["videoName"]
        duration = data["data"]["time"]

        stream = data["data"]["videoSources"]
        vodList = [i for i in stream if i["url"]]
        column = self.column(vodList, "", "name")
        lists = list(column.values())[::-1]
        assert lists, "lists"
        quality = list(column.keys())[::-1]
        show = self.data(quality, p["hd"])
        vodDict = column[show]

        hlsUrl = f'https://disp.titan.mgtv.com{vodDict["url"]}&ver=0.2.21092&chk=b1e323d526786a667cf809f2e4bb2e25&guid=898839756384243712'
        size = int(vodDict["fileSize"])
        if pay:
            hlsUrl = self.sub("arange=\d+", "arange=0", hlsUrl)
        m3u8 = ""
        for z in range(3):
            hlsSource = self.curl(
                {
                    "url": hlsUrl,
                    "headers": {
                        "useragent": "MGTV-iPhone-appstore/5.5.1 (iPhone; iOS 9.3.2; Scale/2.00)",
                        "encoding": "utf8",
                    },
                }
            )
            hlsJson = self.loads(hlsSource)
            if hlsJson["info"].startswith("http"):
                m3u8 = hlsJson["info"]
                break
        self.logging.debug(f"getM3u8: {hlsSource} \r\n")
        assert m3u8, "m3u8"
        extra = {
            "headers": {
                "Accept-Language": "zh-cn",
                "X-Playback-Session-Id": "D11C5818-6A9D-401F-9F8E-E1093AFA2A46",
                "Accept": "*/*",
                "User-Agent": "AppleCoreMedia/1.0.0.16E227 (iPhone; U; CPU OS 12_2 like Mac OS X; zh_cn)",
                "Referer": m3u8,
                "Pragma": "no-cache",
                "Cache-Control": "no-cache",
            },
        }

        ext = playback = "m3u8"

        return self.compact()

    def encode_tk2(self, s):
        string = self.self.replace(
            ["+", "/", "="], ["_", "~", "-"], self.self.b64encode(s)
        )
        return string[::-1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@File    : qq.py
@Time    : 2022/9/3 上午7:24
"""
import template
from Crypto.Cipher import AES
from binascii import hexlify, unhexlify
import time


class Main(template.Template):
    def __init__(self):
        super().__init__()
        self.title = "腾讯视频(QQ)"

    def query(self):
        p = self.params

        if self.hasurl(p["parse"]):
            vid = self.match(
                ["vid=(\w+)", "/(\w+)$", "cover\/\w+\/(\w+)"],
                p["parse"],
            )
            html = self.curl(p["parse"])
            if not vid:
                vid = self.match(
                    [
                        "&vid=(\w+)",
                        "vid:\s*[\"'](\w+)",
                        "vid\s*=\s*[\"']\s*(\w+)",
                        '"vid":"(\w+)"',
                    ],
                    html,
                )
        else:
            vid = p["parse"]
        if p.get("query"):
            info = self.curl(
                {
                    "url": "https://vv.video.qq.com/getinfo",
                    "params": {
                        "charge": "0",
                        "vid": vid,
                        "defaultfmt": "auto",
                        "otype": "json",
                        "guid": "33fe1dbd31182cc02a7fcebb32df167c",
                        "platform": "10901",
                        "defnpayver": "1",
                        "appVer": "3.4.21",
                        "sdtfrom": "v1010",
                        "host": "v.qq.com",
                        "_rnd": self.timestamp,
                        "fhdswitch": "0",
                        "show1080p": "1",
                        "isHLS": "0",
                        "newplatform": "10901",
                        "defsrc": "2",
                        "tm": self.timestamp,
                        "sphttps": "v.qq.com",
                        "spwm": "4",
                        "ehost": f"https://v.qq.com/x/page/{vid}.html",
                    },
                }
            )
            self.logging.debug(f"getInfo: {info} \r\n")
            data = self.jsonParse(info)
            assert "msg" not in data, data["msg"]
            if "lnk" in data["vl"]["vi"][0]:
                vid = data["vl"]["vi"][0]["lnk"]
            title = data["vl"]["vi"][0]["ti"]
            duration = data["vl"]["vi"][0]["td"]

            image = f"http://puui.qpic.cn/vpic_cover/{vid}/{vid}_hz.jpg"

        return self.compact()

    def parse(self):
        p = self.params
        assert p["vid"], "vid"
        timestamp = self.timestamp / 1000
        vid = p["vid"]
        if self.cookie:
            s = self.curl(
                {
                    "url": "https://access.video.qq.com/user/auth_refresh?vappid=11059694&vsecret=fdf61a6be0aad57132bc5cdf78ac30145b6cd2c1470b0cfe&type=wx&g_tk=&g_vstk=854964343&g_actk=87088392&callback=jQuery19109635376764129958_1662439073395&_=1662439073397",
                    "headers": {
                        "Host": "access.video.qq.com",
                        "user-agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:104.0) Gecko/20100101 Firefox/104.0",
                        "accept": "*/*",
                        "accept-language": "zh-CN,zh;q=0.8,zh-TW;q=0.7,zh-HK;q=0.5,en-US;q=0.3,en;q=0.2",
                        "referer": "https://v.qq.com/",
                        "sec-fetch-dest": "script",
                        "sec-fetch-mode": "no-cors",
                        "sec-fetch-site": "same-site",
                        "te": "trailers",
                    },
                    "response": "response",
                }
            )
            # print(s['cookie'])
            if s.get("cookie").get("refresh_token"):
                self.cookie = s.get("cookie")

        # 解析路线
        vUrl = "https://vv.video.qq.com/getvinfo?"
        pUrl = "https://vd.l.qq.com/proxyhttp?"

        mp4 = ""
        platform = "10201"
        sdtfrom = "v1010"
        ctime = self.strftime("%Y-%m-%d %H:%M:%S", timestamp)

        refere = f"https://v.qq.com/x/page/{vid}.html"
        appver = "3.4.31"
        pid = self.md5(vid)
        guid = self.md5(f"{timestamp}{vid}")
        encryptVer = "8.1"
        cKey = self.ckey8(vid, platform, appver, guid, timestamp)
        image = f"http://puui.qpic.cn/vpic_cover/{vid}/{vid}_hz.jpg"

        onlyInfo = self.curl(
            {
                "url": vUrl,
                "params": {
                    "charge": "0",
                    "vid": vid,
                    "defaultfmt": "auto",
                    "otype": "json",
                    "guid": guid,
                    "platform": platform,
                    "defnpayver": "1",
                    "appVer": appver,
                    "sdtfrom": sdtfrom,
                    "host": "v.qq.com",
                    "ehost": refere,
                    "_rnd": timestamp,
                    "defn": "shd",
                    "fhdswitch": "0",
                    "show1080p": "1",
                    "isHLS": "0",
                    "newplatform": platform,
                    "tm": timestamp,
                    "encryptVer": encryptVer,
                    "cKey": cKey,
                    "sphttps": "v.qq.com",
                    "spwm": "4",
                },
                "referer": refere,
            }
        )

        self.logging.debug(f"getVideoInfo: {onlyInfo} \r\n")
        data = self.jsonParse(onlyInfo)
        ip = data["ip"]
        m3u8 = ""
        assert "msg" not in data, data["msg"]
        if "lnk" in data["vl"]["vi"][0]:
            vid = data["vl"]["vi"][0]["lnk"]
            cKey = self.ckey8(vid, platform, appver, guid, timestamp)
        # 判断其是否是会员vip
        drm = data["vl"]["vi"][0]["drm"]
        fvkey = data["vl"]["vi"][0]["fvkey"]
        vu = data["vl"]["vi"][0]["ul"]["ui"][0]["url"]
        fileDict = self.column(data["fl"]["fi"], "", "sname")

        quality = list(fileDict.keys())
        show = self.data(quality, p["hd"])

        stream = fileDict[show]
        fmt = str(stream["id"])
        flowid = f"{pid}_{fmt}"
        defn = stream["name"]
        title = data["vl"]["vi"][0]["ti"]
        if fmt[0:2] == "10":
            fp = f"p{fmt[2:]}"
        elif fmt[0:2] == "11":
            fp = f"p{fmt[1:]}s"
        else:
            fp = fmt
        # 分段数目
        fc = data["vl"]["vi"][0]["cl"]["fc"]
        duration = data["vl"]["vi"][0]["td"]
        pay = 1 if drm else ""

        if "msg" in data:
            assert data["msg"] not in (
                "vid is wrong",
                "vid status wrong",
            ), "hide"

        segs = []
        if not pay:
            fc = fc if fc != 0 else 1
            idxLen = int(float(duration) / 3000) + 1
            array = [[] for k in range(idxLen)]
            for i in range(fc):
                array[int(i / 10)].append(str(i + 1))
            idxs = ["|".join(j) for j in array if len(j)]
            for idx in idxs:

                vkeyUlr = f"{pUrl}buid=onlyvkey&vkeyparam=" + self.quote(
                    self.urlencode(
                        {
                            "otype": "ojson",
                            "vid": vid,
                            "format": fmt,
                            "idx": idx,
                            "vt": "116",
                            "sdtfrom": sdtfrom,
                            "platform": platform,
                            "guid": guid,
                            "flowid": flowid,
                            "charge": "0",
                            "linkver": "2",
                            "lnk": vid,
                            "tm": timestamp,
                            "referer": refere,
                            "ehost": refere,
                            "appVer": appver,
                            "host": "v.qq.com",
                            "sphttps": "1",
                            "encryptVer": encryptVer,
                            "cKey": cKey,
                        }
                    )
                )

                source = self.curl(vkeyUlr)
                self.logging.debug(f"getOnlyvkey: {source} \r\n")
                vSource = self.loads(source)
                vdata = self.loads(vSource["vkey"])
                assert "msg" not in vdata, "data"
                size = int(stream["fs"])
                z = 0
                for i in vdata["vl"]["vi"][0]["cl"]["ci"]:

                    if i.get("key"):
                        filename = f"{vid}.{fp}.{i['idx']}.mp4"
                        guidd = self.md5(f"{guid}{vid}{i['idx']}{ip}{timestamp}")
                        if fc == 1:
                            segs.append(
                                {
                                    "url": f"{vu}{filename}?sdtfrom={sdtfrom}&guid={guidd}&vkey={i['key']}",
                                    "duraion": duration,
                                }
                            )
                        else:
                            segs.append(
                                {
                                    "url": f"{vu}{filename}?sdtfrom={sdtfrom}&guid={guidd}&vkey={i['key']}",
                                    "duraion": data["vl"]["vi"][0]["cl"]["ci"][z]["cd"],
                                }
                            )
                            z += 1
                if len(segs) == 1:
                    mp4 = segs[0]["url"]
        if not mp4:

            try:

                defn = defn if self.cookie else "hd"
                for defn in [defn, "hd"]:

                    oUrl = f"{pUrl}buid=onlyvinfo&vinfoparam=" + self.quote(
                        self.urlencode(
                            {
                                "charge": 0,
                                "guid": guid,
                                "defaultfmt": "auto",
                                "otype": "ojson",
                                "platform": platform,
                                "sdtfrom": sdtfrom,
                                "defnpayver": 1,
                                "appVer": appver,
                                "host": "v.qq.com",
                                "refer": refere,
                                "ehost": refere,
                                "sphttps": 1,
                                "tm": timestamp,
                                "encryptVer": encryptVer,
                                "cKey": cKey,
                                "spwm": 4,
                                "vid": vid,
                                "defn": defn,
                                "fhdswitch": 1,
                                "show1080p": 1,
                                "isHLS": 1,
                                "onlyGetinfo": True,
                                "dtype": 3,
                                "sphls": 1,
                                "defsrc": 2,
                            }
                        )
                    )

                    onlyInfo = self.curl(oUrl)
                    self.logging.debug(f"onlyInfo: {onlyInfo} \r\n")
                    json = self.loads(onlyInfo)
                    if "vinfo" in json:
                        data = self.loads(json["vinfo"])
                        try:
                            turl = data["vl"]["vi"][0]["ul"]["ui"][0]
                            m3u8 = turl["url"] + turl["hls"]["pt"]
                            playback = "m3u8"
                            if len(segs) < 1:
                                segs.append({"url": m3u8, "duration": duration})
                        except:
                            pass

                    if not m3u8:
                        quality = list(fileDict.keys())[:2]
                        show = self.data(quality, p["hd"])
                    else:
                        break
            except:
                mUrl = f"{pUrl}buid=onlyvkey&vkeyparam=" + self.quote(
                    self.urlencode(
                        {
                            "otype": "ojson",
                            "vid": vid,
                            "format": "2",
                            "idx": "0",
                            "vt": "116",
                            "sdtfrom": sdtfrom,
                            "platform": platform,
                            "guid": guid,
                            "flowid": flowid,
                            "charge": "0",
                            "linkver": "2",
                            "lnk": vid,
                            "tm": timestamp,
                            "referer": refere,
                            "ehost": refere,
                            "appVer": appver,
                            "host": "v.qq.com",
                            "sphttps": "1",
                            "encryptVer": encryptVer,
                            "cKey": cKey,
                        }
                    )
                )
                mSource = self.curl(mUrl)
                mVkey = self.match(
                    ['"key\\\\":\\\\"(\w+)', '"key\\":\\"(\w+)'], mSource
                )
                if mVkey:
                    mp4 = "{}{}.mp4?sdtfrom={}&guid={}&vkey={}".format(
                        vu, vid, sdtfrom, guid, mVkey
                    )
                    if len(segs) < 1:
                        segs.append({"url": mp4, "duration": duration})
        if not mp4 and m3u8:
            ext = playback = "m3u8"
        extra = {
            "reload": 1,
            "params": {
                "platform": platform,
                "sdtfrom": sdtfrom,
                "format": fmt,
                "guid": guid,
                "flowid": flowid,
                "appver": appver,
                "encryptVer": encryptVer,
                "cKey": bytes.decode(cKey),
            },
        }
        return self.compact()

    def ckey8(self, vid, platform, appver, guid, tm):
        loc3 = f"|{vid}|{tm}|mg3c3b04ba|{appver}|{guid}|{platform}|https://v.qq.com/x/page/{vid}.html|mozilla/5.0 (macintosh; intel mac os x 10.13; rv|https://v.qq.com/|Mozilla|Netscape|MacIntel|00|"
        loc4 = 0
        i = 0
        while i < len(loc3):
            char = ord(loc3[i])
            loc4 = self.intval32((loc4 << 5) - loc4 + char)
            loc4 &= loc4
            i = i + 1
        loc5 = f"|{loc4}{loc3}"
        # pack H*
        key = unhexlify("4f6bdaa39e2f8cb07f5e722d9edef314")
        iv = unhexlify("01504af356e619cf2e42bba68c3f70f9")
        cipher = AES.new(key, AES.MODE_CBC, iv)
        # 填充
        text_length = len(loc5)
        to_pad = AES.block_size - (text_length % AES.block_size)
        if to_pad == 0:
            to_pad = AES.block_size
        pad = chr(to_pad)
        enc = loc5 + pad * to_pad
        plaintext = cipher.encrypt(enc.encode("utf-8"))
        return hexlify(plaintext).upper()

    def intval32(self, val):
        maxint = 2147483647
        if not -maxint - 1 <= val <= maxint:
            val = (val + (maxint + 1)) % (2 * (maxint + 1)) - maxint - 1
        return val



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@File    : douyin.py
@Time    : 2022/9/11 下午4:03  
"""
import template


class Main(template.Template):
    def __init__(self):
        super().__init__()
        self.title = "抖音直播(DOUYIN)"

    def query(self):
        p = self.params
        if self.hasurl(p["parse"]):

            if "live.douyin.com" in p["parse"]:
                html = self.curl(
                    {
                        "url": p["parse"],
                        "cookie": "__ac_nonce=0731e9abc003e9f166f7;  ",
                    }
                )
                vid = self.match(["room_id%3D(\d+)", "roomId%22%3A%22(\d+)"], html)

            else:
                url = self.curl({"url": p["parse"], "response": "location"})
                vid = self.match("\/(\d+)", url)

        else:
            vid = p["parse"]
        if p.get("query"):
            url = f"https://webcast.amemv.com/webcast/room/reflow/info/?verifyFp=&type_id=0&live_id=1&room_id={vid}&sec_user_id=&app_id=1128&msToken="
            for i in range(3):
                html = self.curl(
                    {
                        "url": url,
                        # 'from':'',
                    }
                )
                if html:
                    break
            data = self.jsonParse(html)
            try:
                data = self.jsonParse(html)
                room = self.haskey(data, "data.room")
                title = room["title"]
                image = room["cover"]["url_list"][0]
                anchor = room["owner"]["nickname"]
            except:
                pass
        return self.compact()

    def parse(self):
        p = self.params
        assert p["vid"], "vid"
        vid = p["vid"]

        # url = f"https://live.douyin.com/{vid}"
        #
        # html = self.curl(
        #     {
        #         "url": url,
        #         "cookie": "__ac_nonce=0731e9abc003e9f166f7;  ",
        #     }
        # )
        # data = self.match(
        #     [
        #         'id="RENDER_DATA" type="application/json">([^\<]+)</script>',
        #         "__INIT_PROPS__\s*=\s*([^\<]+)</script>",
        #     ],
        #     html,
        # )
        # json=self.jsonParse(self.urlParse.unquote(data))
        # # print(json)
        # print(self.haskey(json,'app.initialState.roomStore.roomInfo.room'))
        url = f"https://webcast.amemv.com/webcast/room/reflow/info/?verifyFp=&type_id=0&live_id=1&room_id={vid}&sec_user_id=&app_id=1128&msToken="
        for i in range(3):
            html = self.curl(
                {
                    "url": url,
                    # 'from':'',
                }
            )
            if html:
                break
        data = self.jsonParse(html)
        assert self.haskey(data, "data.room.status") == 2, "close"
        room = self.haskey(data, "data.room")
        title = room["title"]
        image = room["cover"]["url_list"][0]
        anchor = room["owner"]["nickname"]
        stream = room["stream_url"]
        resolution_name = stream["resolution_name"]
        flv_pull_url = stream["flv_pull_url"]
        qualitys = list(flv_pull_url.keys())[::-1]
        qualitys.sort(key=["SD1", "SD2", "HD1", "FULL_HD1", "ORIGION"].index)
        quality = [resolution_name[i] for i in qualitys]
        show = self.data(quality, p["hd"])
        resolution = self.data(qualitys, p["hd"])
        flv = flv_pull_url[resolution]
        m3u8 = stream["hls_pull_url_map"][resolution]
        ext = "flv"
        playback = "m3u8"
        return self.compact()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@File    : iqiyi.py
@Time    : 2022/9/26 下午7:26  
"""
import template


class Main(template.Template):
    def __init__(self):
        super().__init__()
        self.title = "爱奇艺视频(IQIYI)"

    def query(self):
        p = self.params
        if self.hasurl(p["parse"]):
            html = self.curl(p["parse"])
            vid = self.match(
                [
                    'data-player-videoid="(\w+)"',
                    '"vid":"(\w+)"',
                    "param['vid'] = \"(\w+)\"",
                ],
                html,
            )
            cid = self.match(
                [
                    "tvId:(\d+)",
                    'data-player-tvid="(\d+)"',
                    "param['tvid'] = \"(\d+)\"",
                    '"tvid":"(\d+)"',
                    '"tvId":(\d+)',
                ],
                html,
            )
            pay = 1 if self.match('"payMark":(\d+)', html) else 0
        else:
            string = self.replace(["|", ";", ",", "-", "_", ":"], "-", p["parse"])
            spl = string.split("-")
            if len(spl) == 2:
                vid = spl[0] if len(spl[0]) == 32 else spl[1]
                cid = spl[1] if len(spl[0]) == 32 else spl[0]
            elif len(spl[0]) == 32:
                cid = ""
            else:
                cid = spl[0]
        assert cid, "tvid"
        getBaseInfo = self.curl(
            {
                "url": f"https://mac.video.iqiyi.com/video/video/baseinfo/{cid}?src=01082001010000000000",
                "decoding": "utf8",
            },
        )
        self.logging.debug(f"baseInfo: {getBaseInfo} \r\n")
        baseJson = self.loads(getBaseInfo)

        if baseJson.get("data"):
            vid = baseJson["data"]["vid"]
            image = baseJson["data"]["imageUrl"].replace(".jpg", "_480_270.jpg")
            title = baseJson["data"]["name"]
            if "subtitle" in baseJson["data"]:
                title += " " + baseJson["data"]["subtitle"]
            pay = "1" if baseJson["data"]["payMark"] else ""
            duration = self.seconds(baseJson["data"]["duration"])
        else:
            getVideoInfo = self.curl(
                f"https://pcw-api.iqiyi.com/video/video/playervideoinfo?tvid={cid}"
            )
            self.logging.debug(f"videoInfo: {getVideoInfo} \r\n")
            data = self.loads(getVideoInfo)
            assert data.get("data"), "data"
            if not vid:
                source = data["data"]["vu"]
                html = self.curl(source)
                vid = self.match(
                    [
                        'data-player-videoid="(\w+)"',
                        '"vid":"(\w+)"',
                        "param['vid'] = \"(\w+)\"",
                    ],
                    html,
                )
                pay = 1 if self.match('"payMark":(\d+)', html) else 0
            image = data["data"]["vpic"]
            title = data["data"]["vn"]
            if "subt" in data["data"]:
                title += " " + data["data"]["subt"]

        tm = self.timestamp * 1000
        rand = self.md5(tm)
        key = "d5fb4bd9d50c4be6948c97edd7254b0e"
        src = "76f90cbd92f94a2e925d83e8ccd22cb7"
        sc = self.md5(f"{tm}{key}{vid}")
        url = f"https://cache.m.iqiyi.com/tmts/{cid}/{vid}/?sc={sc}&src={src}&t={tm}"
        tmts = self.curl(url)
        tmtsJson = self.loads(tmts)

        if self.haskey(tmtsJson, "data.ctl.configs"):
            configs = tmtsJson["data"]["ctl"]["configs"]
            vids = configs.get("19") or configs.get("10")
            if vids:
                vid = vids["vid"]
        return self.compact()

    def parse(self):
        p = self.params
        assert p["vid"], "vid"
        vid = p["vid"]
        cid = p["cid"]
        if self.haskey(p, "context.tmts"):
            tmts = p["context"]["tmts"]
        else:
            tm = self.timestamp * 1000
            rand = self.md5(tm)
            key = "d5fb4bd9d50c4be6948c97edd7254b0e"
            src = "76f90cbd92f94a2e925d83e8ccd22cb7"
            sc = self.md5(f"{tm}{key}{vid}")
            url = (
                f"https://cache.m.iqiyi.com/tmts/{cid}/{vid}/?sc={sc}&src={src}&t={tm}"
            )
            tmts = self.curl(url)
        self.logging.debug(f"getTmts: {tmts} \r\n")
        json=self.jsonParse(tmts)
        assert json.get("code") == "A00000", "data"
        dicts = {"h265": {}, "h264": {}}
        for i in json["data"]["vidl"]:
            types = "h265" if i.get("fileFormat") == "H265" else "h264"
            dicts[types][i["vd"]] = i
        if p.get("encoder") == "h265":
            lists = dicts["h265"]
        else:

            lists = dicts["h264"]

        lists[json["data"]["vd"]] = {
            "screenSize": json["data"]["screenSize"],
            "m3u": json["data"]["m3u"],
        }
        vs = {int(v["screenSize"].split("x")[0]): k for k, v in lists.items()}

        vsize = sorted(vs.keys())
        quality = [str(vs[i]) for i in vsize]
        show = self.data(quality, p["hd"])
        data = lists[int(show)]
        m3u8 = data["m3u"]
        playback = ext = "m3u8"
        return self.compact()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@File    : __main__.py
@Time    : 2022/9/10 下午1:39
"""
from argparse import ArgumentParser

import coloredlogs, re, importlib, traceback, logging, json, sys, os, traceback, time

sys.path.append(os.path.abspath(os.path.dirname(__file__)))

from main import Parse


def cmd():
    parser = ArgumentParser(
        description="A simple video downloader written in python",
        usage="qito [OPTIONS] URL [URL...]",
    )

    parser.add_argument(
        "-V", "--version", action="store_true", help="Print version and exit"
    )
    parser.add_argument(
        "-c", "--cookie", default="", type=str, help="Cookie [file.txt/Netscape Cookie]"
    )
    parser.add_argument(
        "-e",
        "--exit",
        default=False,
        action="store_true",
        help="End process when parsing playlist error",
    )
    parser.add_argument("-f", "--format", default="", help="Video format")
    parser.add_argument(
        "-i", "--info", default=False, action="store_true", help="Print all information"
    )
    parser.add_argument(
        "-j", "--json", default=False, action="store_true", help="Print json"
    )

    parser.add_argument(
        "-q",
        "--query",
        default=False,
        action="store_true",
        help="Print video information only",
    )
    parser.add_argument(
        "-r", "--hd", type=int, help="Video resolution selection [1,2, ...]"
    )
    parser.add_argument(
        "-s", "--debug", default=False, action="store_true", help="Print  messages"
    )

    parser.add_argument("-t", "--type", help="Parsing type,Add when urls is not a URL")

    parser.add_argument("-v", "--category", help="Site category [video/music/live]")
    parser.add_argument(
        "-x", "--proxy", default=False, help="Proxy option [http/https/socks5]"
    )
    parser.add_argument("-y", "--http", default=False, help="Http proy")
    parser.add_argument(
        "-z",
        "--timeout",
        default=False,
        type=int,
        help="Set httpGet timeout seconds, default 6s",
    )
    playlist_grp = parser.add_argument_group("Playlist options")

    playlist_grp.add_argument(
        "-b",
        "--choose",
        default=False,
        help="Choose which range when the urls is a playlist",
    )
    playlist_grp.add_argument(
        "-l",
        "--playlist",
        default=False,
        action="store_true",
        help="Add when urls is playlist",
    )

    download_grp = parser.add_argument_group("Download options")
    download_grp.add_argument(
        "-d", "--download", default=False, action="store_true", help="Download"
    )

    download_grp.add_argument(
        "-m", "--merge", default=True, action="store_false", help="Do not merge videos"
    )
    download_grp.add_argument(
        "-n",
        "--name",
        default=False,
        type=str,
        help="Video downloaded with the FileName you want",
    )
    download_grp.add_argument(
        "-u",
        "--multi",
        default=False,
        type=int,
        help="Multi-threaded download, default 10",
    )
    download_grp.add_argument("-w", "--dir", default=False, help="Storage folder")
    download_grp.add_argument(
        "-a",
        "--capture",
        default=False,
        action="store_true",
        help=" Download clips only",
    )
    player_grp = parser.add_argument_group("Player options")
    player_grp.add_argument(
        "-g",
        "--geometry",
        default=False,
        help="Adjust the initial window position or size,For example,1366 or 50﹪",
    )
    player_grp.add_argument(
        "-lo",
        "--loop",
        default=False,
        help="Loop",
    )
    player_grp.add_argument(
        "-p",
        "--player",
        default=False,
        help="Directly play the video with PLAYER like mpv",
    )

    player_grp.add_argument(
        "-fs",
        "--fullscreen",
        default=False,
        action="store_true",
        help="Fullscreen playback",
    )
    player_grp.add_argument(
        "-ts", "--start", default=False, help="Seek to given time position"
    )
    player_grp.add_argument(
        "-te", "--end", default=False, help="Stop at given absolute time"
    )
    player_grp.add_argument(
        "-tl",
        "--length",
        default=False,
        help="Stop after a given time relative to the start time",
    )
    player_grp.add_argument(
        "-na",
        "--no_audio",
        default=False,
        action="store_true",
        help="Do not play sound",
    )
    player_grp.add_argument(
        "-nv",
        "--no_video",
        default=False,
        action="store_true",
        help="Do not play video",
    )
    player_grp.add_argument(
        "-vo",
        "--volume",
        default=False,
        help="Volume",
    )
    extra_grp = parser.add_argument_group("Extra options")
    extra_grp.add_argument(
        "-init",
        "--init",
        default=False,
        help="Initial setup, backup or restore user.py",
    )
    extra_grp.add_argument(
        "-o", "--language", default=False, help="Select language options"
    )
    # 额外
    extra_grp.add_argument(
        "-itag", "--itag", default=False, help="Youtube itag, 137[,value]"
    )
    extra_grp.add_argument("-ccode", "--ccode", default=False, help="Youku ccode")
    extra_grp.add_argument("-ip", "--iniPath", default=False, help="Config iniPath")
    extra_grp.add_argument("-fp", "--filePath", default=False, help="Config filePath")
    extra_grp.add_argument(
        "-pwd", "--password", default=False, help="Encryption password for video"
    )
    extra_grp.add_argument(
        "-enc",
        "--encoder",
        default=False,
        help="Select the video option to encode",
    )

    parser.add_argument("urls", type=str, nargs="*", help="video urls")

    args = parser.parse_args()
    if args.version:
        from config.profile import version

        sys.exit(version)
    elif args.capture:
        args.download = True
        if not args.start:
            print("Please set the start time")
            sys.exit()
    return args


def main():
    params = vars(cmd())
    if not params["urls"]:
        sys.exit("usage: qito [OPTION]... URL...")

    # logging.basicConfig(handlers=[log.ColorHandler()])
    FIELD_STYLES = dict(
        asctime=dict(color="green"),
        hostname=dict(color="magenta"),
        levelname=dict(color="green"),
        filename=dict(color="magenta"),
        name=dict(color="red"),
        threadName=dict(color="green"),
    )

    LEVEL_STYLES = dict(
        debug=dict(color="green"),
        info=dict(color="cyan"),
        warning=dict(color="yellow"),
        error=dict(color="red"),
        critical=dict(color="red"),
    )

    coloredlogs.install(
        level="DEBUG",
        fmt="%(levelname)s:%(filename)s:%(funcName)s:%(lineno)d:%(message)s",
        level_styles=LEVEL_STYLES,
        field_styles=FIELD_STYLES,
    )
    if params.get("debug"):
        logging.root.setLevel(logging.DEBUG)
        logging.getLogger("urllib3").setLevel(logging.WARNING)
        logging.getLogger("chardet").setLevel(logging.WARNING)
    else:
        logging.root.setLevel(logging.WARNING)
    try:
        for parse in params["urls"]:
            params["parse"] = parse
            Parse(params)
    except (AssertionError, IndexError) as e:
        info = sys.exc_info()
        try:
            err = traceback.extract_tb(info[2])[1]
        except:
            err = traceback.extract_tb(info[2])[0]
        error = {
            "message": str(e),
            "file": err[0],
            "line": err[1],
            "function": err[2],
            "code": err[3],
        }
        logging.warning(error)
        # logging.error(e.message)
    except (NotImplementedError, KeyError, NameError, AttributeError, TypeError) as e:
        logging.error(e.message)
        if params["exit"]:
            sys.exit()
    except KeyboardInterrupt:
        print("\r\n")
        logging.warning("ctrl c!")
        sys.exit()


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@File    : youku.py
@Time    : 2021/10/19 下午7:10  
"""
import template


class Main(template.Template):
    def __init__(self):
        super(Main, self).__init__()
        self.title = "优酷(YOUKU)"

    def query(self):

        p = self.params
        if self.hasurl(p["parse"]):
            if "v_nextstage" not in p["parse"]:
                url = p["parse"]
            else:
                url = self.curl({"url": p["parse"], "response": "location"})
            vid = self.match(["id_([^\"'\.]+)\.", "sid\/([^\"']+)\/v.swf"], url)
            if vid.isdigit():
                vid = "X" + self.b64encode(str(4 * int(vid)))
        elif p["parse"].isdigit():
            vid = "X" + self.b64encode(str(4 * int(p["parse"])))
        else:
            vid = p["parse"]

        if p.get("query"):
            ccode = p.get("ccode", "0524")
            try:
                utid = self.curl(
                    {
                        "url": "http://log.mmstat.com/eg.js",
                        "headers": {
                            "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.16; rv:84.0) Gecko/20100101 Firefox/84.0",
                            "Referer": "https://g.alicdn.com/alilog/oneplus/blk.html",
                        },
                        "response": "cookie",
                    },
                )["cna"]
            except:
                utid = "qyiUGD8MHWkCARudQQu7gaFM"
            params = {
                "vid": vid,
                "ccode": ccode,
                "client_ip": "192.168.1.1",
                "utid": utid,
                "client_ts": self.timestamp,
                "ckey": "115#1dCil11O1TaNcv6zGfND1Cso311GLyAi1/2Wsi/gdl1it86z19WZy56NE8P1IvvCtU/8yPZQi/WJ1aU4AWNcaLBfOZPQOSAPetT4yWZQgbvJhEz4vBN4+5yAurrQ/jfyet/4yWZQiQ+ghZz8OWNcaTpjurPdvOoNPKxRLp+2i7lL1FGYdPYMTT9cxCNRlGg0kxHucjHqect+1pQNBWMKLRoQD1ZDrSxQ+F6RY5gk0Bmywzgc8WLIgwd2piFbn1q6EWcfZc1Wg5bS6ancr86G4xLB9BSOV9noSrKFv5r4lzKO8RWkfVdXHmIATO1SsV5RtVQCAcMDKsROmJbQy8ozjXN+EK4p3CZSxTcnDRaAr/TrOZSCIXoXN8k3EkUCh1QV3dpRqjRzsM4qBS36s8JOXl9VM4+QJ0+DLRJUKP3J11CNo0odneVsS52H58CYeEqN",
            }

            html = self.curl(
                {
                    "url": "https://ups.youku.com/ups/get.json",
                    "params": params,
                    "referer": f"http://v.youku.com/v_show/id_{vid}.html",
                }
            )
            self.logging.debug(f"getInfo: {html} \r\n")
            data = self.loads(html)
            if "video" in data["data"]:
                pay = data["data"]["show"]["pay"]
                title = data["data"]["video"]["title"]

                image = data["data"]["video"]["logo"]
                duration = data["data"]["video"]["seconds"]

        return self.compact()

    def parse(self):

        p = self.params
        assert p["vid"], "vid"
        vid = p["vid"]
        timestamp = self.timestamp
        utid = ""
        if self.get("cookie"):
            utid = self.match("cna=([^;]+);", self.cookie)
        else:
            try:
                utid = self.curl(
                    {
                        "url": "http://log.mmstat.com/eg.js",
                        "headers": {
                            "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.16; rv:84.0) Gecko/20100101 Firefox/84.0",
                            "Referer": "https://g.alicdn.com/alilog/oneplus/blk.html",
                        },
                        "response": "cookie",
                    },
                )["cna"]
            except:
                pass

        if not utid:
            utid = "qyiUGD8MHWkCARudQQu7gaFM"
        ccode = p.get("ccode") or "0524"

        params = {
            "vid": vid,
            "ccode": ccode,
            "client_ip": "192.168.1.1",
            "utid": utid,
            "client_ts": timestamp,
            "ckey": "099#4a8781f72f29ab250a484c1bf330aa99#82",
            "ptoken": "",
            "stoken": "",
            # "extag": "EXT-X-PRIVINF",
            "needbf": "1",
            "encryptR_client": "MzujwsgiTomoWckrTxQvuw==",
            "key_index": "key01",
            "app_ver": "8.0.5.9181",
            "os_ver": "win_6.1-64",
            "pid": "ywebapp",
            "mac": "94de805741f",
            "rid": "7",
            # "drm_type": "3",
            # "h265": "1",
            "master_m3u8": "1",
            "play_ability": "4352",
            "preferClarity": "99",
            "wintype": "interior",
            "p": "1",
            "appname": "iku",
            "avs": "8.0.5.9181",
            "bt": "pc",
            "aw": "a",
            "site": "1",
            "fu": "0",
            "vs": "1.0",
            "os": "win_6.1-64",
            "rst": "mp4",
            "dq": "intelligent",
            "guid": "100000000000000000005F6601C594DE805741F0",
        }
        html = self.curl(
            {"url": "https://ups.youku.com/ups/get.json", "params": params}
        )
        self.logging.debug(f"getVideo: {html} \r\n")
        json = self.loads(html.replace("http:", "https:"))

        assert "error" not in json, "data"
        # pay = json["data"].get("show").get("pay")
        pay = self.haskey(json, "data.show.pay")
        title = json["data"]["video"]["title"]
        image = json["data"]["video"]["logo"]
        duration = json["data"]["video"]["seconds"]
        lang = []
        if self.haskey(json, "data.dvd.audiolang"):
            language = []
            for i in json["data"]["dvd"]["audiolang"]:

                language.append(
                    {
                        "url": f"https://v.youku.com/v_show/id_{i['vid']}.html",
                        "langcode": i["langcode"],
                        "language": i["lang"],
                    }
                )
                if i["langcode"] == self.params.get("language") and title:
                    title = "{}[{}]".format(title, i["lang"])
            extra = {"language": language}
            lang = self.column(json["data"]["dvd"]["audiolang"], "langcode")

        ary = [
            "3gphd",
            "3gphdv2",
            "flvhd",
            "flvhdv2",
            "mp4sd",
            "mp4sdv2",
            "mp4hd",
            "hd",
            "mp4hdv2",
            "mp4hd2",
            "hd2",
            "mp4hd2v2",
            "mp4hd3",
            "mp4hd3v2",
        ]
        lang = lang or ["ja", "en", "in", "kr", "yue", "guoyu", "default"]
        langDict = {}
        for lk in json["data"]["stream"]:
            if lk["audio_lang"] in langDict:
                langDict[lk["audio_lang"]].append(lk)
            else:
                langDict[lk["audio_lang"]] = []

        encodeid = vid.replace("=", "")
        vidStream = []

        for k, v in langDict.items():
            if encodeid in self.dumps(vid):
                vidStream = v
                break

        # 获取 stream[lang] 数据
        if self.params.get("language") in langDict:
            stream = langDict[self.params["language"]]
        elif vidStream:
            stream = vidStream
        elif "default" in langDict:
            stream = langDict["default"]
        else:
            stream = self.data(langDict, p["hd"])

        streamList = []
        quality = []
        for kn in ary:
            for kb in stream:
                if kb["stream_type"] == kn:
                    streamList.append(kb)
                    quality.append(kb["stream_type"])
        show = self.data(quality, p["hd"])

        fileDict = self.data(streamList, p["hd"])
        segs = []
        for v in fileDict["segs"]:
            if "cdn_url" in v:
                segs.append(
                    {
                        "url": v["cdn_url"],
                        "duration": v["total_milliseconds_video"] / 1000,
                        "size": int(v["size"]),
                    }
                )

        m3u8 = fileDict["m3u8_url"]
        if len(segs) == 0 and m3u8:
            segs = [{"url": m3u8}]
            ext = "m3u8"
        if segs[0]["url"].find(".flv") > 0:
            ext = "flv"
        playback = "m3u8"

        return self.compact()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@File    : qq.py
@Time    : 2022/10/8 上午10:49  
"""
import template


class Main(template.Template):
    def __init__(self):
        super().__init__()
        self.title = "腾讯音乐(QQ)"

    def query(self):
        p = self.params
        if p["parse"].startswith("http"):
            if "songid" in p["parse"]:
                vid = self.match(r"songid=(\w+)", p["parse"])
            elif "song/" in p["parse"]:
                vid = self.match(r"song\/(\w+)", p["parse"])
            elif "songDetail" in p["parse"]:
                vid = self.match(r"songDetail\/(\w+)", p["parse"])

        else:
            vid = p["parse"]
        mParams = {
            "hostUin": "0",
            "format": "jsonp",
            "inCharset": "utf8",
            "outCharset": "utf-8",
            "notice": "0",
            "platform": "yqq",
            "needNewCode": "0",
            "data": '{"comm":{"ct":24,"cv":0},"songinfo":{"method":"get_song_detail_yqq","param":{"song_type":0,"song_mid":"%s"},"module":"music.pf_song_detail_svr"}}'
            % vid,
        }
        url = "https://u.y.qq.com/cgi-bin/musicu.fcg?{}".format(self.urlencode(mParams))
        html = self.curl(url)
        self.logging.debug(f"getInfo: {html} \r\n")
        json = self.loads(html)
        assert json["songinfo"]["data"], "data"
        data = json["songinfo"]["data"]

        company = self.haskey(data, "info.company.content.0.value")
        genry = self.haskey(data, "info.genre.content.0.value")

        album = data["track_info"]["album"]["title"].strip()
        duration = data["track_info"]["interval"]
        cover = data["track_info"]["album"]["mid"]
        image = f"https://y.gtimg.cn/music/photo_new/T002R300x300M000{cover}.jpg"
        title = data["track_info"]["title"].strip()
        cid = data["track_info"]["id"]
        publish = data["track_info"]["time_public"]
        language = data["info"]["lan"]["content"][0]["value"].strip()
        singer = " | ".join([i["title"].strip() for i in data["track_info"]["singer"]])
        return self.compact()

    def parse(self):
        p = self.params
        vid = p["vid"]
        params = {
            "hostUin": "0",
            "format": "jsonp",
            "inCharset": "utf8",
            "outCharset": "utf-8",
            "notice": "0",
            "platform": "yqq",
            "needNewCode": "0",
            "data": '{"req":{"module":"CDN.SrfCdnDispatchServer","method":"GetCdnDispatch","param":{"guid":"7152021848","calltype":0,"userip":""}},"req_0":{"module":"vkey.GetVkeyServer","method":"CgiGetVkey","param":{"guid":"7152021848","songmid":["%s"],"songtype":[0],"uin":"","loginflag":1,"platform":"20"}},"comm":{"uin":"","format":"json","ct":24,"cv":0}}'
            % vid,
        }
        url = "https://u.y.qq.com/cgi-bin/musicu.fcg?{}".format(self.urlencode(params))
        html = self.curl(url)
        self.logging.debug(f"getSong: {html} \r\n")
        json = self.jsonParse(html)
        try:
            m4a = (
                "http://isure.stream.qqmusic.qq.com/"
                + json["req_0"]["data"]["midurlinfo"][0]["purl"]
            )
            # print(m4a)
            playback = ext = "m4a"
        except:
            raise NotImplementedError("data")

        return self.compact()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@File    : le.py
@Time    : 2022/8/28 上午10:51  
"""
import template


class Main(template.Template):
    def __init__(self):
        super().__init__()
        self.title = "乐视剧集列表"

    def videoList(self, params):
        if params["parse"].startswith("http"):
            cover = self.match("\/(\d+)", params["parse"])
        else:
            cover = params["parse"]
        url = f"http://d.api.m.le.com/detail/episode?pid={cover}&platform=pc&page=1&pagesize=2000&type=1&_=1565045426195"
        html = self.curl(url)
        json = self.loads(html)
        assert json["code"] == "200", "playlist"
        lists = [i["vid"] for i in json["data"]["list"]]
        return {"data": lists, "category": "video", "type": "le"}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@File    : col.py
@Time    : 2022/9/5 下午9:45  
"""


class Prepare:
    def prepare_location(self):
        """
        有些链接类型一样,必须通过跳转后地址才能获取正确的category
        :return:
        """
        return ["douyin"]

    def prepare_category(self):
        """
        获取链接类型
        :return:
        """
        lists = {
            "live": {
                "heibaizhibo": ["heibaizhibo.com"],
                "qq": ["(live|now).qq.com"],
                "iqiyi": ["live.iqiyi.com\/(.*)"],
                "pps": ["gamelive.iqiyi.com", "x.pps.tv"],
                "zhuafan": ["www.zhuafan.live\/(\d+)"],
                "douyin": ["webcast.amemv.com\/", "live.douyin.com"],
                "douyu": ["www.douyu.com\/(\d+)", "(douyu.com\/topic)"],
                "bilibili": ["(live.bilibili)"],
                "cctv": ["(cctv.com\/live)"],
                "longzhu": ["(?<!v.)longzhu.com\/(\w+)"],
                "laifeng": ["(v.laifeng.com)"],
                "huya": ["(www.huya.com)"],
                "huajiao": ["(huajiao.com\/l\/)"],
                "yy": ["(yy.com\/\d+\/\d+)"],
                "ixigua": ["(live.ixigua.com)"],
                "56": ["(qf.56.com)"],
                "pptv": ["(sports.pptv.com\/sportslive)"],
                "yizhibo": ["(yizhibo.com)"],
                "zhangyu": ["zhangyu(?:live)*.com\/(\d+)"],
                "huomao": ["huomao.com\/(?:mobile\/mob_live\/)*(\d+)"],
                "chushou": ["chushou.tv\/room/(\d+)"],
                "kuaishou": ["(live.kuaishou.com\/)(?!u\/\w+\/\w+)(?!playback\/)"],
                "twitch": ["(twitch.tv\/)(?!videos\/)(\w+)"],
                "zhanqi": [
                    "(?!videos)",
                    # "(?<!videos.)zhanqi.tv"
                ],
            },
            "music": {
                "bilibili": ["(bilibili.com\/audio)"],
                "qq": ["(y.qq.com)"],
                "netease": ["music.163.com\/(?:m|\#)*(?:\/)*song"],
                "migu": ["(c.migu.cn)", "(h5.nf.migu.cn)"],
                "kuwo": ["(bd.kuwo.cn\/play_detail)"],
                "kugou": ["(kugou.com\/song)"],
                "xiami": ["(xiami.com\/song)"],
                "taihe": ["(music.taihe.com)", "(music.baidu.com)"],
            },
        }
        return lists

    def prepare_change(self, site):
        """
        解析链接的type替换
        :param site:
        :return:
        """
        lists = {
            "douyutv": "douyu",
            "letv": "le",
            "toutiao": "ixigua",
            "iesdouyin": "douyin",
            "iq": "iqiyi",
            "toutiaoimg": "toutiao",
            "ppsport": "pptv",
            "365yg": "toutiao",
            "b23": "bilibili",
            "zhangyulive": "zhangyu",
            "cctv": "cntv",
            "gifshow": "kuaishou",
            "tianmao": "tmall",
            "tb": "taobao",
            "amemv": "douyin",
            "youtu": "youtube",
            "tudou": "youku",
        }

        if site in lists.keys():
            site = lists[site]
        return site

    def prepare_quality(self, params):
        if params["category"] == "live":
            pass
        elif params["category"] == "music":
            pass
        else:
            dicts = {"le": {"350": "流畅", "1000": "标清"}, "youku": {"mp4sd": "标清"}}
        try:

            if self.get(f"{params['category']}Quality") and self.get(
                f"{params['category']}Quality"
            ).get(params["type"]):
                data = dict(
                    k.split(":")
                    for k in (
                        self.get(f"{params['category']}Quality")
                        .get(params["type"])
                        .split("|")
                    )
                )
            else:
                data = dicts[params["type"]]
            quality = params["quality"]
            params["quality"] = [data.get(k, k) if k in data else k for k in quality]
            if quality != params["quality"]:
                params["description"] = quality
            params["show"] = data.get(params["show"], params["show"])
        except:
            pass

        return params

    def prepare_compact(self):
        return [
            "uid",
            "cid",
            "ape",
            "vid",
            "aid",
            "language",
            "page",
            "streams",
            "pay",
            "playback",
            "context",
            "m3u8",
            "segs",
            "duration",
            "anchor",
            "mid",
            "cdn",
            "country",
            "m4a",
            "size",
            "show",
            "title",
            "ext",
            "playtype",
            "flv",
            "mp4",
            "hd",
            "mp3",
            "flac",
            "publish",
            "quality",
            "wav",
            "genre",
            "image",
            "singer",
            "otype",
            "artist",
            "album",
            "cover",
            "company",
            "extra",
            "upload",
        ]

    def prepare_stream(self, lists, stacks, funcName=""):
        r = {}
        if funcName in ["parse", "afresh"]:
            r["streams"] = {}
            if "ext" not in stacks:
                r["ext"] = "mp4"
            if "playback" not in stacks:
                r["playback"] = "mp4"

        if funcName == "extra":
            for index in stacks:
                r[index] = lists[index]
        else:
            for index in stacks:
                if lists[index]:
                    if index in ["quality"]:
                        r[index] = lists[index]
                        r["multirates"] = len(r[index])

                    elif index in [
                        "segs",
                        "live",
                        "music",
                        "mp4",
                        "m3u8",
                        "flv",
                        "mp3",
                        "m4a",
                        "flac",
                        "ape",
                        "m4v",
                        "wav",
                    ]:
                        r["streams"][index] = lists[index]

                    else:
                        r[index] = lists[index]

        if (
            funcName in ["parse", "afresh"]
            and not r["streams"].get("segs")
            and r["streams"].get(r["ext"])
        ):
            r["streams"]["segs"] = [{"url": r["streams"][r["ext"]]}]
            if r.get("duration"):
                r["streams"]["segs"][0]["duration"] = r["duration"]
            if r.get("size"):
                r["streams"]["segs"][0]["size"] = r["size"]

        # 设置extra headers
        if funcName in ["parse", "afresh"] and not r.get("extra"):
            r["extra"] = {
                "headers": {
                    "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.13; rv:68.0) Gecko/20100101 Firefox/68.0"
                }
            }
        elif (
            r.get("extra")
            and not r["extra"].get("remove")
            and not r["extra"].get("headers")
        ):
            r["extra"]["headers"] = {
                "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.13; rv:68.0) Gecko/20100101 Firefox/68.0"
            }
        for k, v in list(r.items()):
            if not v:
                del r[k]
        return r



#!/usr/bin/env python

# -*- coding: utf-8 -*-
"""
@File    : main.py
@Time    : 2022/2/13 下午6:14  
"""
import time, importlib, os, re, logging, sys, requests
from pathlib import Path

from util import common


class Parse(common.Common):
    def __init__(self, params):
        super().__init__()
        self.abspath = os.path.abspath(os.path.dirname(__file__))
        self.cwd = os.getcwd()
        if params.get("debug"):
            logging.basicConfig(
                format="%(levelname)s:%(filename)s:%(funcName)s:%(lineno)d:%(message)s",
                level=logging.DEBUG,
            )

            logging.getLogger("urllib3").setLevel(logging.WARNING)
            logging.getLogger("chardet").setLevel(logging.WARNING)
        else:
            logging.basicConfig(
                format="%(levelname)s:%(filename)s:%(funcName)s:%(lineno)d:%(message)s",
                level=logging.WARNING,
            )

        self.configuration()
        if params["parse"] == "config":
            if not Path(f"{self.abspath}/config/config.py").exists():
                content = 'iniPath=""\nfilePath=""'
                self.write(f"{self.abspath}/config/config.py", content)
                print("Created: config.py created successfully")
            else:
                print("Exists: config.py already exists")

            from config import config

            if params.get("iniPath"):
                config.iniPath = params["iniPath"]
            if params.get("filePath"):
                config.filePath = params["filePath"]

            content = "\n".join(
                [
                    f'{i}="{getattr(config, i, None)}"'
                    for i in [e for e in dir(config) if not e.startswith("_")]
                ]
            )
            self.write(f"{self.abspath}/config/config.py", content)
        elif params["parse"] == "upgrade":
            try:
                file = self.abspath + "/qito-main.zip"
                print("正在下载远程zip...")
                r = requests.get(
                    "https://github.com/qitoqito/qito/archive/refs/heads/main.zip"
                )

                with open(file, "wb") as f:
                    f.write(r.content)
                    f.close()

            except:
                pass
            import zipfile
            import shutil

            zip = zipfile.ZipFile(file)
            print("下载完成,即将解压...")
            for name in zip.namelist():
                zip.extract(name, self.abspath)
            print("正在移动文件...")
            shutil.copytree(
                self.abspath + "/qito-main/qito", self.abspath, dirs_exist_ok=True
            )

        elif params["parse"] == "ini":
            pass
        elif params.get("playlist"):
            self.playList(params)
        else:
            self.working(params)

    def configuration(self):
        self.getConfig("config")
        self.getConfig("profile")
        self.getConfig("user")
        if self.get("iniPath"):
            ini = self.parseIni(f"{self.iniPath}/config.ini")
        else:
            ini = self.parseIni(f"{self.abspath}/ini/config.ini")

    def working(self, params):
        params["category"] = params.get("category") or self.get("category") or "video"
        params["hd"] = params.get("hd") or self.get("hd") or 6
        params["parse"] = str(params["parse"])
        if self.hasurl(params["parse"]):
            site = self.domain(params["parse"])
            if site in self.prepare_location():
                params["parse"] = self.curl(
                    {
                        "url": params["parse"],
                        "response": "location",
                    }
                )
            type = self.prepare_change(site)

            for k, v in self.prepare_category().items():
                if site in v:
                    if self.match(v[site], params["parse"]):
                        params["category"] = k
                        break
                else:
                    for k1, v1 in v.items():
                        if self.match(v1, params["parse"]):
                            type = k1
                            params["category"] = k
                            site = k1
                            break

        else:
            site = params["type"]

            type = self.prepare_change(site)

        if self.get("site") and site in self.siteChange.keys():
            type = self.siteChange[site]

        params["site"] = site
        params["type"] = type
        try:
            imp = importlib.import_module(f"parse.{params['category']}.{type}")
            self.imp = imp
            a = imp.Main()

            a.init(params)
        except ValueError as e:
            print(e)
        except KeyboardInterrupt:
            # ctrl + c 终止运行
            print("\r\n")
            logging.warning("End Process")
            sys.exit()

    def playList(self, params):
        domain = self.match(r"(\w+(?:-\w+)*).\w+\/", params["parse"])

        type = params.get("type") or domain
        category = params.get("category") or "video"
        # try:

        try:
            imp = importlib.import_module(f"parse.playlist.{type}")
            a = imp.Main()
            data = getattr(a, f"{category}List")(params)
            assert len(data["data"]) > 0, "lists"
            params["category"] = data.get("category")
            params["type"] = data.get("type")

            if params.get("choose"):
                params["choose"] = str(params["choose"])
                if ":" in params["choose"]:
                    spl = [i for i in params["choose"].split(":") if i]
                    start = int(spl[0]) - 1

                    if len(spl) > 1:
                        parseLists = data["data"][start : int(spl[1])]
                    else:
                        parseLists = data["data"][start:]

                else:
                    spl = params["choose"].split(",")
                    parseLists = []
                    for i in spl:
                        try:
                            parseLists.append(data["data"][int(i) - 1])
                        except:
                            pass
            else:
                parseLists = data["data"]

        except:
            parseLists = [params["parse"]]
        for i in parseLists:
            if isinstance(i, dict):
                params = {**params, **i}
            else:
                params["parse"] = i

            self.working(params)
        # except:
        #     pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@File    : download.py
@Time    : 2022/8/9 下午6:32  
"""
import time, importlib, os, sys, logging, json, subprocess, threading, requests
from pathlib import Path


class Execute:
    def execute_init(self):
        if not self.data.get("info"):
            self.execute_directory()
        self.execute_console()
        self.block = 12 if sys.platform in ["win32"] else 32  # 终端█显示数目
        self.data["threading"] = []  # 多线程存储
        self.data["output"] = []
        self.data["multi"] = self.data.get("multi") or 10
        self.data["title"] = self.data.get("title") or self.data["title"]
        if self.data.get("info"):
            self.execute_info()
        elif self.data.get("player"):
            self.execute_player()
        else:
            self.execute_download()
        print("\n")

    def execute_directory(self):
        self.data["dir"] = (
            self.data.get("dir")
            or self.get("filePath")
            or f"{self.cwd}/download/{self.data['type']}"
        )
        if self.data["dir"].startswith("."):
            self.data["dir"] = f'{self.cwd}{self.data["dir"][1:]}'
        if not Path(self.data["dir"]).exists():
            os.makedirs(self.data["dir"])

    def execute_console(self):
        if self.data.get("name"):
            self.data["title"] = self.data["name"]
        print("{}:{}{}".format("Web", 17 * " ", self.title))
        for i in [
            "site",
            "title",
            "image",
            "vid",
            "aid",
            "cid",
            "uid",
            "anchor",
            "parse",
            "category",
            "hd",
            "format",
        ]:
            if self.data.get(i):
                print(
                    "{}:{}{}".format(i.capitalize(), (20 - len(i)) * " ", self.data[i])
                )

        print("Stream:")
        print(f"    - Ext:           {self.data['ext']}")
        if self.data.get("playback"):
            print(f"      Playback:      {self.data['playback']}")
        if self.data.get("pay"):
            print(f"      Pay:           {self.data['pay']}")
        if self.data.get("size"):
            size = round(int(self.data["size"]) / 1024 / 1024, 2)
            print(f"      Size:          {size}M [{self.data['size']} Bytes]")
        if self.data.get("duration"):
            print(f"      Duration:      {self.data['duration']}s")
        if self.data.get("quality"):
            if self.data.get("description"):
                print(f"      Description:   {self.data['description']}")
            print(f"      Quality:       {self.data['quality']}")
            print(f"      Show:          {self.data['show']}")
            print(f"      Multirates:    {self.data['multirates']}")

        if self.data["streams"].get("segs"):
            print(f'      Length:        {len(self.data["streams"]["segs"])}')
        if self.data.get("dir"):
            print(f'      Dir:           {self.data["dir"]}')

    def execute_info(self):
        print("Location:")
        print(
            "    - {}:{}{}".format(
                self.data["playback"],
                (14 - len(self.data["playback"])) * " ",
                self.data["streams"][self.data["playback"]],
            )
        )
        if (
            len(self.data["streams"]["segs"]) == 1
            and self.data["ext"] == self.data["playback"]
        ):
            pass
        else:
            for k, v in enumerate(self.data["streams"]["segs"]):
                print(
                    "      part[{}]:{}{} {}".format(
                        (k + 1),
                        (14 - len(f"part[{k + 1}]")) * " ",
                        v["url"],
                        f"[{v.get('size')}]" if v.get("size") else "",
                    )
                )
        if self.data["extra"].get("adaptive"):
            for k, v in enumerate(self.data["extra"]["adaptive"]):
                print(
                    "      itag[{}]:{}{} {}".format(
                        v["itag"],
                        (14 - len(f"part[{v['itag']}]")) * " ",
                        v["url"],
                        f"[{v.get('contentLength')}]" if v.get("contentLength") else "",
                    )
                )
        sys.exit("\r")

    def execute_player(self):
        if self.data["streams"].get("segs") and len(self.data["streams"]["segs"]) == 1:
            self.data["target"] = self.data["streams"]["segs"][0]["url"]
        else:
            self.data["target"] = self.data["streams"][self.data["playback"]]
            # 播放
        if "temp" in [self.haskey(self.data, "extra.player"), self.data["player"]]:
            self.player_temp()
        elif self.data.get("itag") and ":" in self.data["itag"]:
            self.player_itag()
        elif self.data["player"] == "mpv":
            self.player_mpv()

    def execute_download(self):
        if self.data["extra"].get("adaptive"):
            self.download_youtube()
        elif self.data["category"] == "live":
            self.download_live()
        elif self.data["ext"] == "m3u8":
            self.download_m3u8()
        else:
            self.download_segs()

    def execute_export(self):
        cmd = self.cmd
        if self.proxy and self.proxy != "ignore":
            merged = []
            callback = cmd[0]
            del cmd[0]
            for opt in cmd:
                if opt[0:1] == "--":
                    merged.append(opt)
                elif opt[0] in ["-", '"', "'"]:
                    merged.append(opt)
                else:
                    merged.append(repr(opt))
            if sys.platform in ["win32"]:
                export = [
                    "set",
                    f"https_proxy={self.http.replace('socks5', 'https')}",
                    "&&",
                    "set",
                    f"http_proxy={self.http.replace('socks5', 'http')}",
                    "&&",
                    "set",
                    f"all_proxy={self.proxy}",
                    "&&",
                    callback,
                ]
            else:
                export = [
                    "export",
                    f"https_proxy={self.http.replace('socks5', 'https')}",
                    f"http_proxy={self.http.replace('socks5', 'http')}",
                    f"all_proxy={self.proxy}",
                    "&&",
                    callback,
                ]
            call = export + merged
            cmd = " ".join(call).replace("'", '"')
            subprocess.call(cmd, env=self.env, shell=True)
        else:
            subprocess.call(
                cmd,
                env=self.env,
            )

    def player_mpv(self):
        call = ["mpv", self.data["target"]]

        if self.data.get("title"):
            call.extend(
                [
                    f'--title="{self.title}: {self.data["title"].strip()}"',
                    f'--force-media-title="{self.data["title"].strip()}"',
                ]
            )
        if self.data["playback"] in ["m3u8", "hls"]:
            call.extend(
                [
                    "--demuxer-lavf-o="
                    + "protocol_whitelist=[file,http,https,tls,rtp,tcp,udp,crypto,httpproxy]",
                ]
            )
        if self.haskey(self.data, "extra.referer"):
            call.extend(["--referrer=" + self.data["extra"]["referer"]])
        if self.haskey(self.data, "extra.useragent"):
            call.extend(["--user-agent=" + self.data["extra"]["useragent"]])
        if self.haskey(self.data, "extra.headers"):

            extra = self.data["extra"]
            if "Accept-Language" in extra["headers"]:
                del extra["headers"]["Accept-Language"]
            call.extend(
                [
                    '--http-header-fields="{}"'.format(
                        ",".join(
                            ["{}: {}".format(k, v) for k, v in extra["headers"].items()]
                        )
                    )
                ]
            )

        if self.data.get("loop"):
            call.extend(["--loop=" + self.data["loop"]])
        if self.data.get("fullscreen"):
            call.extend(["--fs"])
        if self.data.get("start"):
            call.extend(["--start=" + self.data["start"]])
            if self.data.get("length"):
                call.extend(["--length=" + self.data["length"]])
        if self.data.get("end"):
            call.extend(["--end=" + self.data["end"]])
        if self.data.get("geometry"):
            call.extend(["--geometry=" + self.data["geometry"]])
        if self.data.get("no_video"):
            call.extend(["--no-video"])
        if self.data.get("no_audio"):
            call.extend(["--no-audio"])
        if self.data.get("volume"):
            call.extend(["--volume=" + self.data["volume"]])
        if self.data.get("audio_file"):
            call.extend([f'--audio-file="{self.data["audio_file"]}"'])
        if self.data.get("mime_type"):
            pass

        if self.proxy and self.proxy != "ignore":
            env = os.environ.copy()
            if self.proxy.startswith("socks"):
                call.extend(["--ytdl-raw-options=" + f"proxy=[{self.proxy}]"])
        print("\r")
        print(f"PlayBack: {self.data['target']}")
        self.cmd = call
        self.execute_export()

    def download_segs(self):
        show = f"[{self.data['show']}]" if self.data.get("show") else ""
        self.data["format"] = self.data.get("format") or self.data["ext"]
        self.data["filename"] = f"{self.data['title']}{show}.{self.data['format']}"
        self.data["path"] = f"{self.data['dir']}/{self.data['filename']}"

        if self.data.get("capture"):
            if len(self.data["streams"]["segs"]) == 1:
                self.data["target"] = self.data["streams"]["segs"][0]["url"]
                self.download_format()
            else:
                self.data["target"] = self.data["streams"][self.data["playback"]]
                self.download_format()

            return
        if Path(self.data["path"]).exists():
            self.data["filename"] = self.data["filename"]
            print(f'Exists: {self.data["path"]}', end="")
            return
        self.data["text"] = []
        reload = self.data["extra"].get("reload", 0)

        for k, v in enumerate(self.data["streams"]["segs"]):
            filename = f"{self.data['title']}{show}_{k}.{self.data['ext']}"
            self.data["threading"].append(
                {"filename": filename, "idx": k + 1, "target": v["url"]}
            )
            self.data["text"].append(f"{self.data['dir']}/{filename}")

        self.download_multi()
        # 单段视频,如果指定格式与原本格式一致,跳过FFMPEG转码
        if len(self.data["streams"]["segs"]) == 1 and self.data["ext"] == self.data.get(
            "format"
        ):
            os.rename(f"{self.data['dir']}/{filename}", self.data["path"])
        else:
            self.download_merge()

        print("\r")

    def download_multi(self):
        """
        多线程分段下载
        指定获取self.data["threading"]
        列表子对象dict{'target':'','idx':'','filename':''}
        target:分段链接
        idx:分段编号
        filename:分段存储名

        :param self:
        :return:
        """

        self.data["idxs"] = len(self.data["threading"])
        execute_md5 = []

        for nn in range(100):

            if self.data["ext"] == "m3u8":
                for i in self.data.get("text"):
                    if Path(f"{i}_temp").exists():
                        os.remove(Path(f"{i}_temp"))
                time.sleep(1)
            if nn > 0 and len(execute_md5) < 1:
                break

            threads = []
            # signal.signal(signal.SIGINT, execute_handler)
            # just = len(threading.enumerate())

            if self.data["extra"].get("retry") and nn > 0:
                try:
                    retry = self.imp.parse(self, self.data)
                    self.data = {**self.data, **retry}
                except:
                    pass

            for v in self.data["threading"]:
                if len(execute_md5) > 0:
                    # 网络原因未下载到的分段,在这边会重新加到线程下载
                    if self.md5(v["filename"]) in execute_md5:
                        if self.data["extra"].get("reload"):
                            try:
                                self.data["extra"]["idx"] = v["idx"]
                                self.imp.reload(self)
                                v["target"] = self.data["streams"]["segs"][
                                    v["idx"] - 1
                                ]["url"]
                                r = "\r[%s / %d] |- %s -|  " % (
                                    v["idx"],
                                    self.data.get("idxs", 1),
                                    "reloading",
                                )
                                print("\r{}".format(r), end="")
                            except:
                                pass

                        t = threading.Thread(
                            target=self.download_file,
                            args=(v["target"], v["filename"], v["idx"]),
                        )
                        threads.append(t)

                else:
                    t = threading.Thread(
                        target=self.download_file,
                        args=(v["target"], v["filename"], v["idx"]),
                    )
                    threads.append(t)

            for t in threads:
                t.setDaemon(True)
                t.start()
                num = self.data["multi"]
                if self.data["extra"].get("multi"):
                    num = min(num, self.data["extra"]["multi"])

                while True:
                    if len(threading.enumerate()) <= num:
                        break
            for t in threads:
                t.join()

            #
            # for t in threads:
            #     t.setDaemon(True)
            #     t.start()
            #     num = min(len(threads), self.data["multi"])
            #     while True:
            #         if len(threading.enumerate()) <= num:
            #             break
            # try:
            #
            #     while threads:
            #         for t in threads:
            #             if not t.isAlive():
            #                 threads.remove(t)
            #         time.sleep(0.1)
            # except:
            #     pass

            # threading.BoundedSemaphore(self.data["multi"])
            # for t in threads:
            #     t.setDaemon(True)
            #     t.start()
            #     while True:
            #         if len(threading.enumerate()) <= self.data["multi"]:
            #             break
            #
            # while True:
            #     lock = 0
            #     if len(threading.enumerate()) > just:
            #         lock = 1
            #     if lock:
            #         time.sleep(0.1)
            #     else:
            #         break

            # 获取文件夹中已下载的分段,网络原因导致未下载的分段,循环后重新下载
            # 检查文件名md5,避免文件名包含特殊字符串,正则匹配错误
            time.sleep(0.5)
            files = os.listdir(str(self.data["dir"]))
            files_md5 = [self.md5(i) for i in files]

            execute_md5 = [self.md5(i["filename"]) for i in self.data["threading"]]
            for i in files_md5:
                if i in execute_md5:
                    execute_md5.remove(i)

            if len(execute_md5) < 1:
                break

    def download_file(self, target="", filename="", idx=""):

        """
        :param self:
        :param target: 下载文件链接
        :param filename: 下载文件名
        :param idx: 下载文件编号
        :return:
        """
        target = target or self.data.get("target")
        filename = filename or self.data.get("filename")
        idx = idx or self.data.get("idx", 1)
        if self.data.get("idxs"):
            n = str(idx).zfill(len(str(self.data["idxs"])))
        else:
            n = 1
        path = f'{self.data["dir"]}/{filename}'
        file = Path(path)
        tempname = f"{filename}_temp"
        temp = Path(f"{path}_temp")

        if file.exists():
            r = "\r[%s / %d] |- %s -|  " % (
                n,
                self.data.get("idxs", 1),
                "finish",
            )
            print("\r{}".format(r), end="")
            return
        elif temp.exists():
            temp_size = temp.stat().st_size
        else:
            temp_size = 0

        headers = self.data["extra"]["headers"].copy()
        headers["Range"] = f"bytes={temp_size}-"
        if self.proxy == "ignore":
            proxies = {"http": None, "https": None}
        elif self.proxy:
            if self.proxy.startswith("socks"):
                self.execute_proxy(self.proxy)
                proxies = None
            else:
                proxies = {"http": self.proxy, "https": self.proxy}
        else:
            proxies = None

        try:
            response = requests.get(
                target, stream=True, headers=headers, proxies=proxies
            )
        except:
            try:
                response = requests.get(
                    target, stream=True, headers=headers, proxies=proxies
                )
            except:
                r = "\r[%s / %d] |- %s -|  " % (
                    n,
                    self.data.get("idxs", 1),
                    "ConnectionError",
                )
                print("\r{}".format(r), end="")
                return

        try:
            # 文件剩余大小
            total_size = int(response.headers["Content-Length"])
        except:
            r = "\r[%s / %d] |- %s -|  " % (
                n,
                self.data.get("idxs", 1),
                "contentLengthError",
            )
            print("\r{}".format(r), end="")
            return

        # 文件大小
        if "Content-Range" in response.headers:
            content_range = int(response.headers["Content-Range"].split("/")[1])
        else:
            content_range = total_size

        if content_range == 0:
            return
        # if temp_size > content_range:
        #     os.remove(temp)
        #     temp_size = 0
        # 下载初始时间,参数
        st = time.time()
        start = time.perf_counter()
        temp_data = temp_size

        with open(temp, "ab") as fp:
            for chunk in response.iter_content(chunk_size=1024):
                if chunk:
                    temp_size += len(chunk)
                    mt = time.time()
                    if mt - st > 0.1:
                        speed = (temp_size - temp_data) / 1024 / (mt - st)
                        if speed < 10 and self.data["extra"].get("reload"):
                            return
                        temp_data = temp_size
                        st = mt
                        rate = temp_size / content_range
                        number = int(self.block * rate)
                        r = "\r[%s / %d] |-%s%s-|%d%%  %skb/s %sM/%sM " % (
                            n,
                            self.data.get("idxs", 1),
                            "█" * number,
                            "-" * (self.block - number),
                            int(rate * 100),
                            "%.2f" % speed,
                            "%.2f" % (temp_size / 1024 / 1024),
                            "%.2f" % (content_range / 1024 / 1024),
                        )
                        if self.data.get("download"):
                            print("\r{}".format(r), end="")
                    fp.write(chunk)

        # print("\r")
        # 判断下载文件有效性,多线程m3u8切片太多错误,暂停使用
        if self.data["ext"] == "m3uq8":
            os.rename(temp, path)
        else:
            if os.path.getsize(temp) != 0 and os.path.getsize(temp) == content_range:
                os.rename(temp, path)
            else:
                os.remove(temp)
                time.sleep(0.5)
                return
                # self.retry += 1
                # if self.retry < 10:
                #     os.remove(temp)
                #     # self.retry = 0
                # else:
                #     r = "\r[%s / %d] |- %s -|  " % (
                #         n,
                #         self.data.get("idxs", 1),
                #         "错误过多",
                #     )
                #     print("\r{}".format(r), end="")
                #     return
        # 输出平均速度
        end = time.perf_counter()
        velocity = total_size / 1024 / (end - start)
        r = "\r[%s / %d] |-%s-|%d%%  %skb/s %sM/%sM " % (
            n,
            self.data.get("idxs", 1),
            "█" * self.block,
            100,
            "%.2f" % velocity,
            "%.2f" % (content_range / 1024 / 1024),
            "%.2f" % (content_range / 1024 / 1024),
        )
        print("\r{}".format(r), end="")

    def download_merge(self):
        """
        分段合并
        合并完成后,将删除所有切片或分段
        :param self:
        :return:
        """

        if self.data.get("text"):
            error = 0
            for i in self.data["text"]:
                if not Path(i).exists():
                    error = 1
            if error:
                self.download_multi()
        print(f"\nMerging: video {self.data['filename']} using ffmpeg\n")
        txt = Path(f"{self.data['dir']}/{int(time.time())}.txt")

        with open(str(txt), "w") as fp:
            fp.write("\n".join(["file '{}'".format(i) for i in self.data["text"]]))
        fp.close()
        cmd = [
            "ffmpeg",
            "-hide_banner",
            "-safe",
            "0",
            "-y",
            "-f",
            "concat",
            "-i",
            str(txt),
            "-c",
            "copy",
        ]
        if self.data.get("format") in ["mp4"]:
            cmd.extend(["-bsf:a", "aac_adtstoasc"])

        cmd.extend([self.data["path"]])

        try:
            subprocess.call(
                cmd,
                env=self.env,
            )
            for i in self.data["text"]:
                try:
                    os.remove(i)
                except:
                    pass
            os.remove(txt)
        except:
            logging.error("ffmpeg error!")

    def download_m3u8(self):
        """
        m3u8下载
        playback必须为m3u8
        :param self:
        :return:
        """
        show = f"[{self.data['show']}]" if self.data.get("show") else ""
        self.data["format"] = self.data.get("format", "mp4")
        # ext = "mp4"
        self.data["filename"] = f"{self.data['title']}{show}.{self.data['format']}"
        self.data["path"] = f"{self.data['dir']}/{self.data['filename']}"
        url = self.data["streams"][self.data["ext"]]

        print(f"M3U8: {url}")

        if self.data.get("capture"):
            self.data["target"] = url
            self.download_format()
            return

        if Path(self.data["path"]).exists():
            self.data["filename"] = self.data["filename"]
            print(f'Exists: {self.data["path"]}', end="")
            return

        if (
            Path(f"{self.data['dir']}/{self.data['title']}{show}.mp4")
            and self.data["format"] != "mp4"
        ):
            self.data["input"] = f"{self.data['dir']}/{self.data['title']}{show}.mp4"
            self.download_ffmpeg()
            return

        try:
            html = self.curl({"url": url, "headers": self.data["extra"]["headers"]})

        except:
            try:
                html = self.curl({"url": url, "headers": self.data["extra"]["headers"]})
            except:
                sys.exit("M3u8UrlError")

        if self.data["extra"].get("replace"):
            html = self.replace(
                self.data["extra"]["replace"][0],
                self.data["extra"]["replace"][1],
                html,
            )
        lists = self.matchAll(
            "\#(?:EXTINF|extinf):([^,]+),\s*\r*\n*([^\r|\n|\s]+)", html
        )

        assert lists, "M3u8Error"

        self.data["idxs"] = len(lists)
        ts_lists = [k[1] for k in lists]
        # 如果ts分段非http开头
        url_path = url.rsplit("/", 1)[0] + "/"
        if not ts_lists[0].startswith("http"):
            ts_lists = [f"{url_path}{k}" for k in ts_lists]
        self.data["text"] = []

        for k, v in enumerate(ts_lists):
            filename = f"{self.data['title']}{show}_{k}.ts"
            self.data["threading"].append(
                {
                    "filename": filename,
                    "target": v,
                    "idx": k + 1,
                }
            )
            self.data["text"].append(f"{self.data['dir']}/{filename}")
        self.download_multi()
        self.download_merge()

    def download_ffmpeg(self):
        cmd = [
            "ffmpeg",
            "-i",
            self.data["input"],
            "-vcodec",
            "copy",
            "-acodec",
            "copy",
        ]
        if self.data.get("format") in ["mp4"]:
            cmd.extend(["-bsf:a", "aac_adtstoasc"])

        cmd.extend([self.data["path"]])
        try:
            subprocess.call(
                cmd,
                env=self.env,
            )

        except:
            logging.error("ffmpeg error!")

    def download_live(self):
        """
        直播下载,直接使用ffmpeg录制
        :param self:
        :return:
        """
        show = f"[{self.data['show']}]" if self.data.get("show") else ""

        if self.data["ext"] == "hls":
            ext = self.data.get("format") or "mp4"
            self.data["target"] = self.data["streams"]["m3u8"]
            self.data["filename"] = f"{self.data['title']}{show}.{ext}"

        else:
            ext = self.data.get("format") or self.data["ext"]
            rtime = time.strftime("%Y-%m-%dT%H-%M-%S", time.localtime())
            self.data["target"] = self.data["streams"][self.data["ext"]]
            self.data["filename"] = (
                f"{self.data['title']} - {self.data['anchor']}{show} - {rtime}.{ext}"
                if self.data.get("anchor")
                else f"{self.data['title']}{show} - {rtime}.{ext}"
            )
        self.data["path"] = f"{self.data['dir']}/{self.data['filename']}"

        if Path(self.data["path"]).exists():
            pass
            # sys.exit("已下载")

        headers = []
        if self.data["extra"].get("headers"):
            for k, v in self.data["extra"]["headers"].items():
                headers.append("-headers")
                headers.append(f"{k}:{v}")

        cmd = [
            "ffmpeg",
            "-hide_banner",
            "-protocol_whitelist",
            "file,http,https,tls,rtp,tcp,udp,crypto,httpproxy",
        ]
        if headers:
            cmd.extend(
                [
                    "-user_agent",
                    self.data["extra"]["headers"].get("User-Agent")
                    or self.data["extra"]["headers"].get("user-agent"),
                ]
            )
            cmd.extend(headers)
        cmd.extend(["-i", self.data["target"]])
        cmd.extend(
            [
                "-c:v",
                "copy",
                "-c:a",
                "copy",
                "-bsf:a",
                "aac_adtstoasc",
                self.data["path"],
            ]
        )
        self.cmd = cmd
        self.execute_export()

    def download_youtube(self):
        """
        Youtube下载
        全局变量itag:当以a:b形式时,a和b不允许同时为video或audio,下载后自动ffmpeg合并
            当以a(,b(,c))形式存在时,将会下载a(b(c))分段
            当为video或者audio时,会下载所有itag为video或者audio的分段
            当为all时,会下载所有itag
        不存在全局变量itag,将下载指定分辨率的mp4资源
        :param self:
        :return:
        """

        if self.data.get("itag"):
            if ":" in self.data["itag"] or self.data["itag"] in ["large"]:
                lists = self.column(self.data["extra"]["adaptive"], "", "itag")
                if self.data["itag"] in ["large"]:
                    itags = []
                    lar = {"video": {}, "audio": {}}
                    for kk, vv in lists.items():
                        if "video" in vv["mimeType"] and "avc1" in vv["mimeType"]:
                            lar["video"][int(vv["contentLength"])] = kk
                        elif "audio/webm" in vv["mimeType"]:
                            lar["audio"][int(vv["contentLength"])] = kk
                    itags.append(lar["video"][sorted(lar["video"].keys())[-1]])
                    itags.append(lar["audio"][sorted(lar["audio"].keys())[-1]])
                else:
                    itags = self.data["itag"].split(":")
                print("Itag:", itags)
                cmd = ["ffmpeg", "-hide_banner"]
                self.data["idxs"] = 2
                audio = video = ""
                for k, v in enumerate(itags):
                    info = lists[int(v)]
                    mtype = self.match("(\w+)\/(\w+)", info["mimeType"])
                    filename = f"{self.data['title']}_itag_{v}.{mtype[1]}"
                    self.data["output"].append(filename)
                    if mtype[0] == "audio":
                        audio = v
                    else:
                        video = v

                    self.data["threading"].append(
                        {
                            "filename": filename,
                            "target": info["url"],
                            "idx": k + 1,
                            "ext": mtype[1],
                        }
                    )

                assert audio and video, "video or audio"

                if self.data.get("capture"):
                    timelength = self.data.get("length")
                    start = self.seconds(self.data["start"])
                    end = self.seconds(self.data["end"]) if self.data.get("end") else ""

                    if timelength:
                        t = f"[{self.data['start']}-{timelength}]"
                    else:
                        t = (
                            f"[{self.data['start']}x{self.data['end']}]"
                            if end
                            else f"[{start}]"
                        )

                    self.data["output"] = []
                    # 实验性,用ffmpeg录制itag片段
                    ext = self.data.get("format") or "mp4"

                    merge_name = f"{self.data['title']}_merge_{video}_{audio}{t}.{ext}"

                    if Path(f'{self.data["dir"]}/{merge_name}').exists():
                        print("File exists skip Download!")
                    else:
                        for i in self.data["threading"]:
                            self.data["target"] = i["target"]
                            self.data["filename"] = i["filename"]
                            self.data["format"] = i["ext"]

                            self.download_format()
                        del self.data["target"]
                        merge_name = f"{self.data['title']}_merge_{video}_{audio}{self.data['substr']}.{ext}"

                    self.data["format"] = ext
                    for i in self.data["output"]:
                        cmd.extend(["-i", f'{self.data["dir"]}/{i}'])

                else:
                    merge_name = f"{self.data['title']}_merge_{video}_{audio}.{self.data.get('format', 'mp4')}"
                    if Path(f"{self.data['dir']}/{merge_name}").exists():
                        print("File exists skip Download!")
                    else:
                        self.download_multi()
                    for i in self.data["threading"]:
                        cmd.extend(["-i", f'{self.data["dir"]}/{i["filename"]}'])

                self.data["filename"] = merge_name

                cmd.extend(
                    [
                        "-strict",
                        "-2",
                        "-acodec",
                        "copy",
                        "-vcodec",
                        "copy",
                        f"{self.data['dir']}/{merge_name}",
                    ]
                )

                if not Path(f"{self.data['dir']}/{merge_name}").exists():

                    subprocess.call(
                        cmd,
                        env=self.env,
                    )
                    if self.data.get("capture"):
                        # if True:
                        for i in self.data["output"]:
                            try:
                                os.remove(Path(f'{self.data["dir"]}/{i}'))
                            except:
                                pass
                else:
                    print("ffmpeg ok!")
            else:

                lists = self.column(self.data["extra"]["adaptive"], "", "itag")

                if self.data["itag"] == "all":
                    itags = list(lists.keys())
                elif self.data["itag"] == "audio":
                    itags = [i for i in lists if "audio" in lists[i]["mimeType"]]
                elif self.data["itag"] == "video":
                    itags = [i for i in lists if "video" in lists[i]["mimeType"]]
                else:
                    itags = self.data["itag"].split(",")

                self.data["idxs"] = len(itags)
                for k, v in enumerate(itags):
                    try:
                        info = lists[int(v)]
                        mtype = self.match("(\w+)\/(\w+)", info["mimeType"])
                        self.data["threading"].append(
                            {
                                "filename": f"{self.data['title']}_itag_{v}.{mtype[1]}",
                                "target": info["url"],
                                "idx": k + 1,
                            }
                        )
                    except:
                        pass

                self.download_multi(self)

        else:
            show = f"[{self.data['show']}]" if self.data.get("show") else ""
            self.data["filename"] = f"{self.data['title']}{show}.{self.data['ext']}"
            self.data["target"] = self.data["streams"][self.data["ext"]]
            self.data["path"] = f"{self.data['dir']}/{self.data['filename']}"
            self.download_file()

    def download_format(self):
        """
        视频转码或截取视频片段
        :param self:
        :return:
        """
        start = end = change = temp = ""
        timelength = self.data.get("length")
        self.data["format"] = self.data.get("format") or "mp4"
        if self.data.get("start"):
            start = self.seconds(self.data["start"])
            end = self.seconds(self.data["end"]) if self.data.get("end") else ""

            if timelength:
                t = f"[{self.data['start']}-{timelength}]"
            else:
                t = (
                    f"[{self.data['start']}x{self.data['end']}]"
                    if end
                    else f"[{start}]"
                )
            # 设置文件名
            output = self.sub(
                ".(\w+)$", f"{t}.{self.data['format']}", self.data["filename"]
            )
            self.data["output"].append(output)
            self.data["substr"] = t
        else:
            output = self.sub(
                ".(\w+)$", f".{self.data['format']}", self.data["filename"]
            )

        if not Path(f"{self.data['dir']}/{output}").exists():

            if self.data.get("capture"):
                cmd = [
                    "ffmpeg",
                    "-hide_banner",
                    "-user_agent",
                    self.data["extra"]["headers"].get("User-Agent")
                    or self.data["extra"]["headers"].get("user-agent"),
                    "-protocol_whitelist",
                    "file,http,https,tls,rtp,tcp,udp,crypto,httpproxy",
                ]
                # M3U8内容替换
                if self.data["playback"] == "m3u8" and self.data["extra"].get(
                    "replace"
                ):
                    try:
                        html = self.curl(
                            {
                                "url": self.data["target"],
                                "headers": self.data["extra"]["headers"],
                            }
                        )

                        html = self.replace(
                            self.data["extra"]["replace"][0],
                            self.data["extra"]["replace"][1],
                            html,
                        )
                        temp = f"{self.data['dir']}/{int(time.time())}-{self.data['vid']}-{self.data['type']}.m3u8"
                        with open(temp, "w") as fp:
                            fp.write(html)
                        fp.close()
                        change = self.data["target"]
                        self.data["target"] = temp
                        cmd = [
                            "ffmpeg",
                            "-hide_banner",
                            "-protocol_whitelist",
                            "file,http,https,tls,rtp,tcp,udp,crypto,httpproxy",
                        ]

                    except:
                        pass

                else:
                    headers = []
                    for k, v in self.data["extra"]["headers"].items():
                        headers.append("-headers")
                        headers.append(f"{k}:{v}")

                    cmd.extend(headers)

                cmd.extend(["-i", self.data["target"]])

                if self.data.get("format") == "mp4":
                    cmd.extend(["-bsf:a", "aac_adtstoasc"])

            else:
                cmd = [
                    "ffmpeg",
                    "-i",
                    f"{self.data['dir']}/{self.data['filename']}",
                    "-strict",
                    "-2",
                ]

            if start:
                cmd.extend(["-ss", str(start)])

                if timelength:
                    cmd.extend(["-t", str(timelength)])
                elif end:
                    cmd.extend(["-to", str(end)])

                cmd.extend(["-c", "copy"])

            cmd.append(f"{self.data['dir']}/{output}")

            print(f"\r视频转码中")
            self.cmd = cmd
            call = self.execute_export()

            if temp:
                os.remove(Path(temp))
                self.data["target"] = change
        else:
            print(f"\r视频转码完成")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@File    : template.py
@Time    : 2022/02/13 下午4:52
"""

import time, importlib, os, sys, logging, json

from util import common

from pathlib import Path


class Template(common.Common):
    def __init__(self):
        super().__init__()
        self.title = "QITOER"

        self.proxy = ""
        self.cookie = ""
        self.abspath = os.path.abspath(os.path.dirname(__file__))
        self.cwd = os.getcwd()

    def init(self, params):
        self.params = params
        self.getConfig("config")
        self.getConfig("profile")
        # 处理proxy代理模块
        if params.get("proxy"):
            self.proxy = params["proxy"]

        self.run()

    def run(self):
        parse = {}
        if self.get("iniPath"):
            ini = self.parseIni(f'{self.iniPath}/{self.params["type"]}.ini')
            config = self.parseIni(f"{self.iniPath}/config.ini")

        else:
            ini = self.parseIni(f'{self.abspath}/ini/{self.params["type"]}.ini')
            config = self.parseIni(f"{self.abspath}/ini/config.ini")

        # for k, v in ini.items():
        #     self.set(k, v)
        # for k, v in config.items():
        #     self.set(k, v)

        if self.params.get("cookie"):
            self.cookie = self.params["cookie"]
        if self.params.get("proxy"):
            self.proxy = self.params["proxy"]

        if self.get("import"):
            self.include()
        try:
            query = self.query()
        except:
            query = {}

        try:
            if self.params.get("query"):
                parse = {**self.params, **query}

            else:

                self.params = {**self.params, **query}
                parse = self.parse()
                parse = {**self.params, **parse}

            self.data = parse
            if self.data.get("title"):
                self.data["title"] = self.sub(
                    "[a-zA-Z0-9’!\"#$%&'()*+,-./:;<=>?@，。?★、…【】《》？“”‘’！[\\]^_`{|}~\s]+",
                    "",
                    self.data["title"],
                )
            if "show" in parse:
                self.prepare_quality(parse)
            if self.params.get("download") and not self.params.get("query"):
                self.execute_init()
            elif self.params.get("player") and not self.params.get("query"):
                self.execute_init()
            elif self.params.get("info") and not self.params.get("query"):
                self.execute_init()
            elif self.params.get("json"):
                if self.get("jsonFilter"):
                    filter = self.jsonFilter.split("|")
                    parse = dict([k, v] for k, v in parse.items() if k not in filter)
                print(json.dumps(parse, indent=2, ensure_ascii=False))
            else:
                parse["code"] = 0
                if self.get("jsonFilter"):
                    filter = self.jsonFilter.split("|")
                    parse = dict([k, v] for k, v in parse.items() if k not in filter)
                print(parse)
        except AssertionError as e:
            print(e)

    def compact(self):
        func = sys._getframe(1).f_code.co_name
        getLocals = sys._getframe(1).f_locals
        # params = getattr(self, f"prepare_{func}")(self.params["category"])
        params = self.prepare_compact()
        inter = set(params).intersection(set(getLocals.keys()))
        return self.prepare_stream(getLocals, inter, func)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@File    : setup.py
@Time    : 2022/9/27 下午8:54  
"""
# !/usr/bin/env python3
# -*- coding:UTF-8 -*-
from setuptools import setup

import os, codecs, platform
from qito.config.profile import version

REQ = ["requests", "pysocks", "coloredlogs", "pyexecjs"]


def find_packages(*tops):
    packages = []
    for d in tops:
        for root, dirs, files in os.walk(d, followlinks=True):
            if "__init__.py" in files:
                packages.append(root)
        # packages.extend(['vparse/temp',"vparse/cookie"])
    return packages


setup(
    name="qito",
    version=version,
    author="KEDAYA",
    author_email="ii@iippcc.com",
    url="https://github.com/qitoqito/qito",
    license="MIT",
    description="视频音乐下载播放器",
    packages=find_packages("qito"),
    package_data={"qito": ["parse/json/*.json", "parse/javascript/*.*"]},
    python_requires=">=3.5",
    long_description="""基于Python,视频、音乐、直播下载器,支持YouKu、QQ、Iqiyi等""",
    classifiers=["License :: OSI Approved :: MIT License"],
    platforms="any",
    requires=REQ,
    install_requires=REQ,
    entry_points={
        "console_scripts": [
            "qito=qito.__main__:main",
        ]
    },
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@File    : profile.py
@Time    : 2022/2/13 下午4:48
"""
version = "1.0.4"
jsonFilter = "version|cookie|dir|exit|format|info|json|query|debug|proxy|http|timeout|download|merge|multi|dir|capture|geometry|loop|player|fullscreen|start|end|length|no_audio|no_video|volume|init|language|itag|ccode|password|encoder|name"



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@File    : youku.py
@Time    : 2022/9/4 下午5:01  
"""
import template


class Main(template.Template):
    def __init__(self):
        super().__init__()
        self.title = "优酷剧集列表"

    def videoList(self, params):
        url = params["parse"]
        if url.startswith("http"):
            vid = self.match(["id_([^\"'\.]+)\.", "sid\/([^\"']+)\/v.swf"], url)
        else:
            vid = url

        s = self.curl(
            {
                "url": f"https://search.youku.com/api/search?appScene=show_episode&showIds={vid}&appCaller=h5"
            }
        )
        data = self.loads(s)

        playlist = self.column(data["serisesList"], "videoId")
        return {"data": playlist, "category": "video", "type": "youku"}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@File    : sohu.py
@Time    : 2022/9/27 下午12:14  
"""
import template


class Main(template.Template):
    def __init__(self):
        super().__init__()
        self.title = "搜狐视频(SOHU)"

    def query(self):
        p = self.params
        if p["parse"].isdigit():
            vid = p["parse"]
        else:
            html = self.curl(p["parse"])
            vid = self.match(
                [
                    'data-vid="(\d+)"',
                    "\s+vid:(\d+),",
                    'var\s+vid="(\d+)"',
                    "vid:\s*'(\d+)'",
                    "vid\s*=\s*\\\\'(\d+)\\\\'",
                ],
                html,
            )
        assert vid, "vid"
        for i in range(2):
            site = int(i) + 1
            getInfo = self.curl(
                f"https://api.tv.sohu.com/v4/video/info/{vid}.json?api_key=f351515304020cad28c92f70f002261c&plat=17&site={site}"
            )
            data = self.loads(getInfo)
            if "data" in data:
                break
        self.logging.debug(f"getInfo: {getInfo} \r\n")
        assert "errorCode" not in data, data["statusText"]

        otype = "tv" if site == 1 else "my"
        title = data["data"]["video_name"]
        image = data["data"]["hor_w16_pic"]
        duration = data["data"]["total_duration"]

        context = {}
        ary = {"nor": "nor", "high": "hig", "super": "sup", "original": "original"}
        context["bytes"] = {
            ary[i]: data["data"]["clips_bytes_" + i]
            for i in ["nor", "high", "super", "original"]
            if "clips_bytes_" + i in data["data"]
        }
        return self.compact()

    def parse(self):
        p = self.params
        vid = p["vid"]
        timestamp = self.timestamp

        # h5_cc.gif上报,数据不卡顿
        muid = timestamp * 100 + 98
        uid = timestamp * 1000 + 99
        reportUrl = f"https://z.m.tv.sohu.com/h5_cc.gif?t={muid}&uid={uid}&position=play_fail_cookie&op=click&details=%7B%7D&nid=&url=http%253A%252F%252F127.0.0.1%252Fsohu%252Ftest.html&refer=&screen=1366x768&os=other&platform=Win32&passport=&vid={vid}&pid=1231223&channeled=1212130002&MTV_SRC=11050001"
        self.curl(reportUrl)

        if p.get("oytpe"):
            sites = [p["otype"]]
        else:
            sites = ["tv", "my"]

        for i in sites:
            site = 2 if i == "my" else 1
            playParams = {
                "vid": vid,
                "site": site,
                "appid": "tv",
                "api_key": "f351515304020cad28c92f70f002261c",
                "plat": "17",
                "sver": "1.0",
                "partner": "1",
                "uid": uid,
                "muid": muid,
                "_c": "1",
                "pt": "1",
                "qd": "680",
                "src": "11050001",
                "aid": "1231223",
            }
            api = "https://m.tv.sohu.com/phone_playinfo"
            getSource = self.curl({"url": api, "params": playParams}).replace(
                "http:", "https:"
            )
            data = self.loads(getSource)
            if "data" in data:
                break
        self.logging.debug(f"getVideo: {getSource} \r\n")
        assert "errorCode" not in data, data["statusText"]
        title = data["data"]["video_name"]
        image = data["data"]["hor_w16_pic"]
        duration = data["data"]["total_duration"]

        ary = ["nor", "hig", "sup", "original"]

        d = []
        e = []
        f = []
        segs = []
        for j in ary:
            if self.haskey(data, "data.urls.mp4.%s" % j):
                if len(data["data"]["urls"]["mp4"][j]) > 0:
                    d.append(j)
            if self.haskey(data, "data.urls.m3u8.%s" % j):
                if len(data["data"]["urls"]["m3u8"][j]) > 0:
                    e.append(j)
            if j in data["data"]["durations"]:
                if len(data["data"]["durations"][j]) > 0:
                    f.append(j)
        assert d, "lists"
        quality = d
        show = self.data(quality, p["hd"])
        mp4Data = data["data"]["urls"]["mp4"][show]
        durData = data["data"]["durations"][show]

        try:
            bt = p["context"]["bytes"][show].split(",")
            for i in tuple(zip(mp4Data, durData, bt)):
                segs.append(
                    {
                        "url": self.replace("\d+\.\d+\.\d+\.\d+", "data.vod.itc.cn", i[0]),
                        "duration": i[1],
                        "size": int(i[2]),
                    }
                )

        except:
            for i in tuple(zip(mp4Data, durData)):
                segs.append(
                    {
                        "url": self.sub("\d+\.\d+\.\d+\.\d+", "data.vod.itc.cn", i[0]),
                        "duration": i[1],
                    }
                )
        m3u8 = (
            data["data"]["urls"]["m3u8"][self.data(e, p["hd"])][0] + "&oth=&cd=&prod=h5"
        )
        mp4 = (
            data["data"]["urls"]["downloadUrl"][0][0]
            + "&qd=68001&src=11050001&ca=4&cateCode=101&_c=1&appid=tv&oth=&cd=&prod=h5"
        )
        playback = "m3u8"
        # 可能存在上报出错,重新上报一次
        self.curl(reportUrl)
        extra = {"replace": ["http:", "https:"]}
        return self.compact()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@File    : common.py
@Time    : 2021/10/19 下午4:51  
"""

import re, requests, json, base64, time, hashlib, urllib, logging, os, logging, importlib, urllib3, random, execjs

from util import prepare, execute


class Common(execute.Execute, prepare.Prepare):
    def __init__(self):
        self.n = 0
        self.log = 1
        self.message = []
        self.path = os.path.abspath(os.path.dirname(__file__))
        self.cwd = os.getcwd()
        self.time = time.time()
        self.options = {"header": {}}
        self.modules = {}
        self.env = None
        self.time = int(time.time())
        self.timestamp = int(round(self.time * 1000))
        self.logging = logging
        self.urlParse = urllib.parse

    def set(self, k, v):
        setattr(self, k, v)

    def get(self, key):
        return getattr(self, key, None)

    def include(self):
        for i in self.require:
            try:
                self.modules[i] = importlib.import_module(f"util.{i}")
            except:
                self.modules[i] = importlib.import_module(i)

    def haskey(self, data, key, value="", message=""):
        """
        判断多维数组data中有没有指定key键,或者键值是否与value一致
        :param data:
        :param key:
        :param value:
        :return:
        """
        for i in key.split("."):
            i = int(i) if i.isdigit() else i
            try:
                # data=data.get(i)
                data = data[i]
            except:
                if message:
                    raise AttributeError(message)
                else:
                    return ""
        if value:
            data = True if value == data else False
        return data

    def hasurl(self, link):
        """
        判断是不是url
        :param link:
        :return: true or false
        """
        return link.lower().startswith("http")

    def curl(self, params):
        data = ""
        json = ""
        if isinstance(params, str):
            params = {"url": params}
        response = params.get("response", None)
        method = params.get("method")
        module = requests.post if method == "post" else requests.get
        if params.get("form"):
            module = requests.post
            data = params["form"]
        elif params.get("json"):
            module = requests.post
            json = params["json"]
        verify = True if params.get("verify") else False
        if not verify:
            requests.packages.urllib3.disable_warnings()
        if self.get("proxy"):
            proxies = {
                "http": self.proxy,
                "https": self.proxy,
            }
        else:
            proxies = {}
        if params.get("headers"):
            headers = params["headers"]
        elif self.get("headers"):
            headers = self.headers
        else:
            headers = {}
        if (
            self.get("cookie")
            and not headers.get("Cookie")
            and not headers.get("cookie")
        ):
            if type(self.cookie) == dict:
                self.cookie = ";".join(
                    [" " + k + "=" + str(v) for k, v in self.cookie.items()]
                ).strip()
            headers["Cookie"] = self.cookie
        elif params.get("cookie"):
            if type(params["cookie"]) == dict:
                params["cookie"] = ";".join(
                    [" " + k + "=" + str(v) for k, v in params["cookie"].items()]
                ).strip()
            headers["Cookie"] = params["cookie"]

        if params.get("ua"):
            if params["ua"] == "ios":
                headers[
                    "User-Agent"
                ] = "Mozilla/5.0 (iPhone; CPU iPhone OS 12_0_1 like Mac OS X) AppleWebKit/604.1.34 (KHTML, like Gecko) CriOS/74.0.3729.155 Mobile/15E148 Safari/604.1"
            elif params["ua"] == "android":
                headers[
                    "User-Agent"
                ] = "Mozilla/5.0 (Linux; Android 9; Mi A1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/74.0.3729.157 Mobile Safari/537.36"
            elif params["ua"] == "pc":
                headers[
                    "User-Agent"
                ] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.13; rv:67.0) Gecko/20100101 Firefox/67.0"
            if params["ua"] == "wechat":
                headers[
                    "User-Agent"
                ] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_4) AppleWebKit/605.1.15 (KHTML, like Gecko) MicroMessenger/2.3.25(0x12031910) MacWechat Chrome/39.0.2171.95 Safari/537.36 NetType/WIFI WindowsWechat"
            else:
                headers["User-Agent"] = params["ua"]
        if not headers.get("user-agent") or not headers.get("User-Agent"):
            headers[
                "User-Agent"
            ] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/94.0.4606.81 Safari/537.36"
        if params.get("referer"):
            headers["Referer"] = params["referer"]
        args = dict(
            proxies=proxies,
            headers=headers,
            verify=False,
            timeout=params.get("timeout", 3000),
            params=params.get("params"),
        )
        if data:
            args["data"] = data
        elif json:
            args["json"] = json
        if response == "location":
            c = requests.head(
                params["url"],
                **args,
            )
            if c.headers.get("location"):
                location = c.headers["location"]

            else:
                d = module(params["url"], **args)
                location = d.url or params["url"]
            return location

        elif response == "cookie":
            c = module(params["url"], **args)
            return c.cookies.get_dict()
        elif response == "response":
            html = {}
            c = module(params["url"], **args)
            if params.get("nobody"):
                html["content"] = ""
            else:
                if params.get("decoding"):
                    c.decoding = params["decoding"]
                elif params.get("encoding"):
                    c.encoding = params["encoding"]

                html["content"] = c.text
            co = c.cookies.get_dict()

            if not co and c.headers.get("Set-Cookie"):
                d = self.replace(["; ", ", "], "&", c.headers["Set-Cookie"])
                co = dict(urllib.parse.parse_qsl(d))
                try:
                    del co["path"]
                    del co["expires"]
                except:
                    pass

            html["cookie"] = co

            html["headers"] = dict(c.headers)
            return html
        else:
            c = module(params["url"], **args)

            if response == "json":
                return c.json()
            else:
                return c.text

    def match(self, pattern, string, flags=0):
        """
        正则获取,只取一组数据
        :param pattern:
        :param string:
        :param flags:
        :return:
        """
        if flags == "S":
            flags = re.S
        elif flags == "I":
            flags = re.I
        elif flags == "L":
            flags = re.L
        elif flags == "M":
            flags = re.M
        elif flags == "U":
            flags = re.U
        elif flags == "X":
            flags = re.X
        else:
            flags = 0
        if not isinstance(pattern, list):
            pattern = [pattern]

        for m in pattern:
            try:
                fin = re.search(m, string, flags)
            except TypeError:
                fin = re.search(m, str(string), flags)
            if fin:
                if len(fin.groups()) > 1:
                    r = fin.groups()
                else:
                    try:
                        r = fin.group(1)
                    except:
                        r = fin.group(0)
                return r

        return ""

    def matchAll(self, pattern, string, flags=0):
        """
        正则匹配,获取全部数据
        :param pattern:
        :param string:
        :param flags:
        :return:
        """
        if flags == "S":
            flags = re.S
        elif flags == "I":
            flags = re.I
        elif flags == "L":
            flags = re.L
        elif flags == "M":
            flags = re.M
        elif flags == "U":
            flags = re.U
        elif flags == "X":
            flags = re.X
        else:
            flags = 0
        if not isinstance(pattern, list):
            pattern = [pattern]
        for m in pattern:
            try:
                fin = re.findall(m, string, flags)
            except TypeError:
                fin = re.findall(m, str(string), flags)
            if fin:
                return fin
        return []

    def loads(self, string):
        return json.loads(string)

    def dumps(self, lists, ensure=1):
        return json.dumps(lists, ensure_ascii=ensure)

    def data(self, params, n, key=""):
        """
        排序数组,获取相关键(n-1)的值
        :param params:
        :param n:
        :param key:
        :return:
        """
        num = int(n)
        length = len(params)
        if isinstance(params, dict):
            params = list(params.values())
        else:
            params = list(params)
        if num > length:
            array = [""] * (num - length)
            array.extend(params)
            params = array
        return params[num - 1][key] if key else params[num - 1]

    def domain(self, link):
        """
        获取域名前缀
        :param url:
        :return:
        """
        # v.qq.com
        # qq.com
        # v.qq.com.cn
        # qq-qq.com
        return self.match(["\w+\.(\w+)\.\w+.\w+\/", "(\w+(?:-\w+)*).\w+\/"], link)

    def build(self, s, f="&", t=0):
        if t:
            pass
        else:
            a = urllib.parse.urlencode(s)
            return a.replace(r"&", f)

    def md5(self, string):
        m2 = hashlib.md5()
        m2.update(str(string).encode("utf-8"))
        return m2.hexdigest()

    def sha1(self, string):
        return hashlib.sha1(str(string).encode("utf-8")).hexdigest()

    def sha256(self, string):
        return hashlib.sha256(str(string).encode("utf-8")).hexdigest()

    def column(self, lists, value, key=""):
        """
        参考PHP-column函数,返回数组中指定的一列说明
        :param lists:
        :param value:
        :param key:
        :return:
        """
        lists = lists.values() if type(lists) == dict else lists
        if key:

            try:
                if type(value) == list:
                    ary = {
                        i[key]: {j: i[j] for j in value} if value else i
                        for i in lists
                        if key in i
                    }

                else:
                    ary = {i[key]: i[value] if value else i for i in lists if key in i}
            except:
                ary = {}

        else:
            # ary = [i[value] if value else i for i in lists]
            ary = [i[value] for i in lists if value in i]
        return ary

    def urlencode(self, params):
        return urllib.parse.urlencode(params)

    def strftime(self, format, timestamp=""):
        return time.strftime(format, time.localtime(timestamp))

    def unquote(self, params):
        return urllib.parse.unquote(params)

    def quote(self, params):
        return urllib.parse.quote(params)

    def parseIni(self, iniPath):
        try:
            with open(iniPath) as fileObj:
                contents = fileObj.read()
            text = f"[ENV]{contents}[EOF]"
            dict = {}
            cookies = {}
            self.params = self.get("params") or {"category": ""}
            for i in self.matchAll(r"\[\s*(\w+)\s*\]([^\[]+)", text):
                col = [x for x in i[1].split("\n") if x]
                if "cookie" in i[0]:
                    dict[i[0]] = col
                elif i[0]:
                    dict[i[0]] = {}
                    for k in col:
                        d = self.match(r"(\w+)\s*=\s*([^\n]+)\s*", k)
                        dict[i[0]][d[0]] = d[1]
                else:
                    for k in col:
                        d = self.match(r"(\w+)\s*=\s*([^\n]+)\s*", k)
                        dict[d[0]] = d[1]

            if dict.get(self.params["category"]):
                for k, v in dict[self.params["category"]].items():
                    dict[k] = v
            if dict.get("cookie"):
                if type(dict["cookie"]) is list:
                    dict["cookies"] = dict["cookie"]
                    dict["cookie"] = random.choice(dict["cookies"])
                else:
                    dict["cookies"] = [dict["cookie"]]
            for k, v in dict.items():
                self.set(k, v)
            return dict
        except:
            return {}

    def getConfig(self, name):
        try:
            config = importlib.import_module(f"config.{name}")
            dict = {}
            for i in dir(config):
                if not i.startswith("__"):
                    self.set(i, getattr(config, i))
                    dict[i] = getattr(config, i)
            self.set(f"{name}Space", dict)
        except:
            pass

    def read(self, path, mode="r"):
        fp = open(path, mode)
        data = fp.read()
        fp.close()
        return data

    def write(self, path, data, mode="w"):
        fp = open(path, mode)
        data = fp.write(data)
        fp.close()
        return data

    def replace(self, pattern, repl, string):
        """
        文本替换
        :param pattern:
        :param repl:
        :param string:
        :return:
        """
        if isinstance(pattern, list):
            if isinstance(repl, list):
                for i in range(len(pattern)):
                    string = string.replace(pattern[i], repl[i])
            else:
                for i in range(len(pattern)):
                    string = string.replace(pattern[i], repl)
        else:
            string = string.replace(pattern, repl)
        return string

    def sub(self, pattern, repl, string, count=0, flags=0):
        """
        正则替换
        :param pattern:
        :param repl:
        :param string:
        :param count:
        :param flags:
        :return:
        """
        if isinstance(pattern, list):
            if isinstance(repl, list):
                for i in range(len(pattern)):
                    string = re.sub(pattern[i], repl[i], string, count=0, flags=0)
            else:
                for i in range(len(pattern)):
                    string = re.sub(pattern[i], repl, string, count=0, flags=0)
        else:
            string = re.sub(pattern, repl, string, count=0, flags=0)
        return string

    def jsonParse(self, s):
        try:
            s = json.loads(s)
        except:
            getTry = self.match(
                ["^try\s*\{\s*\n*\s*(\w+)", "^(\w+)\s*\n*\s*\(", "^(\w+\s*=)\s*\{"], s
            )
            if getTry:
                c = execjs.compile(
                    """
                    function kk() {
                            return %s;
                        }
                        """
                    % s.replace(getTry, "cbData=")
                )
                s = c.call("kk")
                # e = quickjs.Function(
                #     "kk",
                #     """
                #     function kk() {
                #         return %s;
                #     }
                #     """
                #     % s.replace(getTry, "cbData="),
                # )
                #
                # s = e()
        return s

    def typeOf(self, variate, extra=""):
        type = None
        if isinstance(variate, int):
            type = "int"
        elif isinstance(variate, str):
            type = "str"
        elif isinstance(variate, float):
            type = "float"
        elif isinstance(variate, list):
            type = "list"
        elif isinstance(variate, tuple):
            type = "tuple"
        elif isinstance(variate, dict):
            type = "dict"
        elif isinstance(variate, set):
            type = "set"
        if extra:
            return type == extra
        else:
            return type

    def seconds(self, tm):
        spl = tm.split(":")
        count = len(spl)
        if count == 1:
            duration = int(tm)

        else:
            if count == 2:
                spl.insert(0, 0)
            try:
                duration = int(spl[0]) * 3600 + int(spl[1]) * 60 + int(spl[2])
            except:
                duration = int(spl[0]) * 3600 + int(spl[1]) * 60 + float(spl[2])

        return duration

    def unique(self, array):
        return list(set(array))

    def getArray(self, name):
        value = self.get(name)
        if value:
            if isinstance(value, list):
                return value
            else:
                return value.split("|")
        else:
            return []



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@File    : bilibili.py
@Time    : 2022/10/7 下午1:10  
"""
import template


class Main(template.Template):
    def __init__(self):
        super().__init__()
        self.title = "哔哩哔哩视频(BILIBILI)"

    def query(self):
        p = self.params
        aid = ""
        page = 1
        vid = ""
        if p["parse"].startswith("av"):
            p["parse"] = "https://www.bilibili.com/video/%s" % p["parse"]
        if p["parse"].startswith("BV"):
            p["parse"] = "https://www.bilibili.com/video/%s" % p["parse"]
        elif p["parse"].startswith("ep"):
            p["parse"] = "https://www.bilibili.com/bangumi/play/%s" % p["parse"]
        if p["parse"].startswith("http"):
            url = self.curl({"url": p["parse"], "response": "location"})
            if "bangumi" in url:

                html = self.curl(
                    {
                        "url": url,
                        "useragent": "ios",
                        "encode": "gzip",
                        "encoding": "utf8",
                    }
                )
                vid = self.match(
                    [
                        # "(\d+)-\d+-\d+.m4s",
                        "\d+\/(\d+)-.+?\.(?:mp4|flv)",
                        # '"cid":(\d+)',
                        "cid=(\d+)",
                        'cid="(\d+)',
                    ],
                    html,
                )
                if vid == "0":
                    vid = ""
                if not vid:
                    loadded = self.match("window.__INITIAL_STATE__=(\{.*?\});", html)
                    if loadded:
                        loadJson = self.loads(loadded)
                        vid = loadJson["epInfo"]["cid"]
                        if vid == -1:
                            vid = loadJson["epList"][0]["cid"]

                title = self.match(
                    [
                        '"share_copy"\s*:\s*"([^"]+)"',
                        '<h1 title="(.*?)">',
                        '"h1Title":"(.*?)"',
                        '"title":"(.*?)"',
                    ],
                    html,
                ).replace("\u002F", "/")
                image = self.match('property="og:image" content="([^"]+)"', html)
                if image:
                    image = image.replace("\u002F", "/")
            elif self.match("\/BV", url):
                bvid = self.match("\/(BV\w+)", url)
                bvUrl = f"http://api.bilibili.com/x/web-interface/view?bvid={bvid}"
                page = self.match(["\?p=(\d+)", "_(\d+).h"], url) or 1
                page = int(page)
                getBvSource = self.curl(bvUrl)
                self.logging.debug(f"getBvSource: {getBvSource} \r\n")
                bvJson = self.loads(getBvSource)
                if bvJson["code"] == 0:
                    aid = bvJson["data"]["aid"]
                    image = bvJson["data"]["pic"]
                    title = bvJson["data"]["title"]
                    cidData = bvJson["data"]["pages"][page - 1]
                    vid = cidData["cid"]
                    if "part" in cidData and page > 1:
                        title = "%s--%s" % (title, cidData["part"])
                else:
                    aid = self.dec(bvid)

            elif "video" in url:
                aid = self.match("\/video\/av(\d+)", url)
                page = self.match(["_(\d+).h", "\?p=(\d+)"], url) or 1

        if aid and not vid:
            page = int(page)
            viewUrl = "http://api.bilibili.com/view?appkey=12737ff7776f1ade&batch=1&page={}&id={}".format(
                page, aid
            )
            getViewSource = self.curl(viewUrl)

            self.logging.debug(f"getInfo: {getViewSource} \r\n")
            if "Document is not exists" in getViewSource:
                raise NotImplementedError("hide")
            elif "Access denied" in getViewSource:
                raise NotImplementedError("cookie")
            else:
                data = self.loads(getViewSource)
                vid = data["cid"]
                image = data["pic"]
                title = data["title"]

                if "partname" in data and page > 1:
                    title = "{}--{}".format(title, data["partname"])
        return self.compact()

    def parse(self):
        timestamp = self.timestamp
        p = self.params
        assert p["vid"], "vid"
        vid = p["vid"]
        token = ""
        if self.cookie:
            getToken = self.curl(
                f"https://api.bilibili.com/x/player/playurl/token?cid={vid}&aid=91275169"
            )
            tokenJson = self.loads(getToken)
            token = self.haskey(tokenJson, "data.token")

        # sk = {"appkey": "YvirImLGlLANCLvM", "appsecret": "JNlZNgfNGKZEpaDTkCdPQVXntXhuiJEM"}
        # sk = {"appkey": "bb3101000e232e27", "appsecret": "36efcfed79309338ced0380abd824ac1"}
        # sk = {"appkey": "07da50c9a0bf829f", "appsecret": "25bdede4e1581c836cab73a48790ca6e"}
        # sk = {"appkey": "4409e2ce8ffd12b8", "appsecret": "59b43e04ad6965f34319062b478f83dd"}
        sk = {
            "appkey": "iVGUTjsxvpLeuDCf",
            "appsecret": "aHRmhWMLkdeMuILqORnYZocwMBpMEOdt",
        }

        otype = "video"
        qn = 80

        mParams = [
            ("appkey", sk["appkey"]),
            ("build", "4140"),
            ("buvid", "ef5efdc0452641eb235cd424dfa03660"),
            ("cid", vid),
            ("device", "phone"),
            ("otype", "json"),
            ("platform", "html5"),
            ("qn", qn),
            ("type", "mp4"),
        ]
        mEncrypt = self.urlencode(mParams)
        mUrl = "https://app.bilibili.com/v2/playurl?%s&sign=%s" % (
            mEncrypt,
            self.md5(mEncrypt + sk["appsecret"]),
        )

        mSource = self.curl(mUrl)

        mJson = self.loads(mSource)

        if "message" in mJson:
            if mJson["message"] not in ["Video is hidden."]:
                raise NotImplementedError(mJson["message"])
            else:
                otype = "bangumi"

                mParams = [
                    ("cid", vid),
                    ("module", "bangumi"),
                    ("otype", "json"),
                    ("platform", "html5"),
                    ("player", "1"),
                    ("qn", qn),
                    ("ts", timestamp),
                    ("type", "mp4"),
                ]
                mEncrypt = self.urlencode(mParams)
                mSign = self.md5(mEncrypt + "9b288147e5474dd2aa67085f716c560d")

                mUrl = (
                    "https://bangumi.bilibili.com/player/web_api/playurl?%s&sign=%s"
                    % (
                        mEncrypt,
                        mSign,
                    )
                )
                mSource = self.curl(mUrl)
                mJson = self.loads(mSource)

        self.logging.debug(f"mSource: {mSource} \r\n")

        assert "8986943" not in mSource, "area"
        assert "durl" in mJson, "lists"

        mp4 = mJson["durl"][0]["url"]

        flvQn = 80
        for i in range(2):
            if otype == "video":
                flvParams = [
                    ("appkey", sk["appkey"]),
                    ("build", "4140"),
                    ("buvid", "ef5efdc0452641eb235cd424dfa03660"),
                    ("cid", vid),
                    ("device", "phone"),
                    ("otype", "json"),
                    ("platform", "html5"),
                    ("qn", flvQn),
                    ("quality", flvQn),
                    ("type", "flv"),
                ]

                flvEncrypt = self.urlencode(flvParams)

                flvUrl = "https://interface.bilibili.com/v2/playurl?%s&sign=%s" % (
                    flvEncrypt,
                    self.md5(flvEncrypt + sk["appsecret"]),
                )
                if token:
                    flvUrl += f"&utoken={token}"
            else:
                flvParams = [
                    ("cid", vid),
                    ("module", "bangumi"),
                    ("otype", "json"),
                    ("platform", "html5"),
                    ("player", "1"),
                    ("qn", flvQn),
                    ("ts", timestamp),
                    ("type", "flv"),
                ]
                flvEncrypt = self.urlencode(flvParams)
                flvSign = self.md5(flvEncrypt + "9b288147e5474dd2aa67085f716c560d")

                flvUrl = (
                    "https://bangumi.bilibili.com/player/web_api/playurl?%s&sign=%s"
                    % (flvEncrypt, flvSign)
                )

            getFlvSource = self.curl(flvUrl)
            getFlvJson = self.loads(getFlvSource)
            qns = sorted(getFlvJson["accept_quality"])

            flvQn = self.data(qns, p["hd"])
            if flvQn >= qn and not self.cookie:
                break
        self.logging.debug(f"getFlvSource: {getFlvSource} \r\n")

        duration = getFlvJson["timelength"] / 1000
        if "url" in getFlvJson["durl"]:
            segs = [
                {
                    "url": getFlvJson["durl"]["url"],
                    "duration": duration,
                    "size": int(getFlvJson["durl"]["size"]),
                }
            ]
        else:
            segs = [
                {
                    "url": i["url"],
                    "duration": i["length"] / 1000,
                    "size": int(i["size"]),
                }
                for i in getFlvJson["durl"]
            ]
        ext = "flv" if segs[0]["url"].find(".flv") > 0 else "mp4"

        quality = getFlvJson["accept_description"][::-1]
        dicts = dict(
            zip(getFlvJson["accept_quality"], getFlvJson["accept_description"])
        )
        quality = [dicts[i] for i in qns]
        # 1800p+分辨率要会员cookie
        regx = f"-1-{flvQn}"
        if p["hd"] > 4 and regx not in segs[0]["url"] and not self.cookie:
            quality = quality[:4]
        show = self.data(quality, p["hd"])

        extra = {
            "headers": {
                "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.13; rv:71.0) Gecko/20100101 Firefox/71.0",
                "Referer": "https://www.bilibili.com/video/",
            }
        }
        if len(segs) == 1 and ext == "flv":
            flv = segs[0]["url"]
            extra["playback"] = "flv"
        return self.compact()

    def dec(x):
        # https://www.zhihu.com/question/381784377/answer/1099438784
        tr = {
            "f": 0,
            "Z": 1,
            "o": 2,
            "d": 3,
            "R": 4,
            "9": 5,
            "X": 6,
            "Q": 7,
            "D": 8,
            "S": 9,
            "U": 10,
            "m": 11,
            "2": 12,
            "1": 13,
            "y": 14,
            "C": 15,
            "k": 16,
            "r": 17,
            "6": 18,
            "z": 19,
            "B": 20,
            "q": 21,
            "i": 22,
            "v": 23,
            "e": 24,
            "Y": 25,
            "a": 26,
            "h": 27,
            "8": 28,
            "b": 29,
            "t": 30,
            "4": 31,
            "x": 32,
            "s": 33,
            "W": 34,
            "p": 35,
            "H": 36,
            "n": 37,
            "J": 38,
            "E": 39,
            "7": 40,
            "j": 41,
            "L": 42,
            "5": 43,
            "V": 44,
            "G": 45,
            "3": 46,
            "g": 47,
            "u": 48,
            "M": 49,
            "T": 50,
            "K": 51,
            "N": 52,
            "P": 53,
            "A": 54,
            "w": 55,
            "c": 56,
            "F": 57,
        }
        s = [11, 10, 3, 8, 4, 6]
        xor = 177451812
        add = 8728348608
        r = 0
        for i in range(6):
            r += tr[x[s[i]]] * 58**i
        return (r - add) ^ xor

    def enc(x):
        table = "fZodR9XQDSUm21yCkr6zBqiveYah8bt4xsWpHnJE7jL5VG3guMTKNPAwcF"
        s = [11, 10, 3, 8, 4, 6]
        xor = 177451812
        add = 8728348608
        x = (int(x) ^ xor) + add
        r = list("BV1  4 1 7  ")
        for i in range(6):
            r[s[i]] = table[x // 58**i % 58]
        return "".join(r)
